#
# Copyright (C) 2009, 2011 by Jonathan Woithe
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from PyQt4 import QtGui

from PyQt4.QtCore import SIGNAL, SLOT, QObject, Qt, QTimer
from PyQt4.QtGui import QWidget, QApplication
from ffado.config import *

from ffado.widgets.matrixmixer import MatrixMixer

import logging
log = logging.getLogger('rme')

# Model defines.  These must agree with what is used in rme_avdevice.h.
RME_MODEL_NONE      = 0x0000
RME_MODEL_FF800     = 0x0001
RME_MODEL_FF400     = 0x0002

class Rme(QWidget):
    def __init__(self,parent = None):
        QWidget.__init__(self,parent)
        uicLoad("ffado/mixer/rme", self)

        self.init()

    def init(self):

        self.PhantomSwitches={
            self.phantom_0: ['/Control/Phantom', 0],
            self.phantom_1: ['/Control/Phantom', 1],
            self.phantom_2: ['/Control/Phantom', 2],
            self.phantom_3: ['/Control/Phantom', 3],
        }

        self.Switches={
            self.ff400_chan3_opt_instr: ['/Control/Chan3_opt_instr'],
            self.ff400_chan3_opt_pad:   ['/Control/Chan3_opt_pad'],
            self.ff400_chan4_opt_instr: ['/Control/Chan4_opt_instr'],
            self.ff400_chan4_opt_pad:   ['/Control/Chan4_opt_pad'],

            self.spdif_output_optical:  ['/Control/SPDIF_output_optical', 0],
            self.spdif_output_emphasis: ['/Control/SPDIF_output_emphasis', 0],
            self.spdif_output_pro:      ['/Control/SPDIF_output_pro', 0],
            self.spdif_output_nonaudio: ['/Control/SPDIF_output_nonaudio', 0],
        }

        self.Radiobuttons={
            self.level_in_lo_gain: ['/Control/Input_level', 1],
            self.level_in_p4dBu:   ['/Control/Input_level', 2],
            self.level_in_m10dBV:  ['/Control/Input_level', 3],

            self.level_out_hi_gain: ['/Control/Output_level', 1],
            self.level_out_p4dBu:   ['/Control/Output_level', 2],
            self.level_out_m10dBV:  ['/Control/Output_level', 3],

            self.spdif_input_coax:    ['/Control/SPDIF_input_mode', 0],
            self.spdif_input_optical: ['/Control/SPDIF_input_mode', 1],

            self.phones_hi_gain: ['/Control/Phones_level', 1],
            self.phones_p4dBu:   ['/Control/Phones_level', 2],
            self.phones_m10dBV:  ['/Control/Phones_level', 3],

            self.clock_mode_autosync: ['/Control/Clock_mode', 1],
            self.clock_mode_master: ['/Control/Clock_mode', 0],

            self.sync_ref_wordclk: ['/Control/Sync_ref', 0],
            self.sync_ref_adat1: ['/Control/Sync_ref', 1],
            self.sync_ref_adat2: ['/Control/Sync_ref', 2],
            self.sync_ref_spdif: ['/Control/Sync_ref', 3],
            self.sync_ref_tco: ['/Control/Sync_ref', 4],
        }

        self.Checkboxes={
            self.ch1_instr_fuzz: ['/Control/Chan1_instr_opts', 0x04],
            self.ch1_instr_limiter: ['/Control/Chan1_instr_opts', 0x08],
            self.ch1_instr_filter: ['/Control/Chan1_instr_opts', 0x02],
        }

        self.Gains={
            self.gain_mic1: ['/Control/Gains', 0],
            self.gain_mic2: ['/Control/Gains', 1],
            self.gain_input3: ['/Control/Gains', 2],
            self.gain_input4: ['/Control/Gains', 3],
        }

        self.Combos={
            self.ff800_ch1_src: ['/Control/Chan1_source'],
            self.ff800_ch7_src: ['/Control/Chan7_source'],
            self.ff800_ch8_src: ['/Control/Chan8_source'],
        }

        # Other mixer variables
        self.is_streaming = 0
        self.sample_rate = 0
        self.model = 0
        self.tco_present = 0

    # Public slot: update phantom power hardware switches
    def updatePhantomSwitch(self, a0):
        sender = self.sender()
        # Value is the phantom switch value, with a corresponding enable
        # bit in the high 16 bit word
        val = (a0 << self.PhantomSwitches[sender][1]) | (0x00010000 << self.PhantomSwitches[sender][1])
        log.debug("phantom switch %d set to %d" % (self.PhantomSwitches[sender][1], a0))
        self.hw.setDiscrete(self.PhantomSwitches[sender][0], val)

    # Public slot: update generic switches
    def updateSwitch(self, a0):
        sender = self.sender()
        log.debug("switch %s set to %d" % (self.Switches[sender][0], a0))
        self.hw.setDiscrete(self.Switches[sender][0], a0)

    # Public slot: update generic radiobuttons
    def updateRadiobutton(self, a0):
        sender = self.sender()
        if (a0 != 0):
            # Only change the control state on a button being "checked"
            log.debug("radiobutton group %s set to %d" % (self.Radiobuttons[sender][0], self.Radiobuttons[sender][1]))
            self.hw.setDiscrete(self.Radiobuttons[sender][0], self.Radiobuttons[sender][1])

    def updateCheckboxes(self, a0):
        sender = self.sender()
        val = self.hw.getDiscrete(self.Checkboxes[sender][0]);
        if (a0 != 0):
            val = val | self.Checkboxes[sender][1]
        else:
            val = val & ~self.Checkboxes[sender][1]
        log.debug("checkbox group %s set to %d" % (self.Checkboxes[sender][0], val));
        self.hw.setDiscrete(self.Checkboxes[sender][0], val)

    # Public slot: update gains
    def updateGain(self, a0):
        sender = self.sender()
        log.debug("gain %s[%d] set to %d" % (self.Gains[sender][0], self.Gains[sender][1], a0))
        self.hw.setMatrixMixerValue(self.Gains[sender][0], 0, self.Gains[sender][1], a0)

    def updateBandwidthLimit(self, a0):
        # Account for the "No ADAT-2" item which will not be present on
        # a FF400.
        if (self.model==RME_MODEL_FF400 and a0>0):
            a0 = a0 + 1
        # log.debug("limit update: %d" % (a0));
        self.hw.setDiscrete('/Control/Bandwidth_limit', a0);

    def updateCombo(self, a0):
        sender = self.sender()
        log.debug("combo %s set to %d" % (self.Combos[sender][0], a0))
        self.hw.setDiscrete(self.Combos[sender][0], a0)

    def updateStreamingState(self):
        ss = self.streamingstatus.selected()
        ss_txt = self.streamingstatus.getEnumLabel(ss)
        if ss_txt != 'Idle':
            self.is_streaming = True
        else:
            self.is_streaming = False
        if (self.last_streaming_state != self.is_streaming):
            self.bandwidth_limit.setEnabled(not(self.is_streaming));
        self.last_streaming_state = self.is_streaming

    def status_update(self):
        # log.debug("timer event")
        self.updateStreamingState()
        clk_mode = ['Master', 'Slave']
        src_str = ['None', 'ADAT 1', 'ADAT 2', 'SPDIF', 'Wordclock', 'TCO']
        sync_stat = ['No lock', 'Locked', 'Synced']
        sysclock_mode = self.hw.getDiscrete('/Control/sysclock_mode')
        sysclock_freq = self.hw.getDiscrete('/Control/sysclock_freq')
        autosync_freq = self.hw.getDiscrete('/Control/autosync_freq')
        autosync_src = self.hw.getDiscrete('/Control/autosync_src')
        sync_status = self.hw.getDiscrete('/Control/sync_status')
        spdif_freq = self.hw.getDiscrete('/Control/spdif_freq')
        self.sysclock_freq.setText("%d Hz" % (sysclock_freq))
        self.sysclock_mode.setText(clk_mode[sysclock_mode])
        self.autosync_freq.setText("%d Hz" % (autosync_freq))
        self.autosync_src.setText(src_str[autosync_src])
        self.sync_check_adat1_status.setText(sync_stat[sync_status & 0x03])
        self.sync_check_adat2_status.setText(sync_stat[(sync_status >> 2) & 0x03])
        self.sync_check_spdif_status.setText(sync_stat[(sync_status >> 4) & 0x03])
        self.sync_check_wclk_status.setText(sync_stat[(sync_status >> 6) & 0x03])
        self.sync_check_tco_status.setText(sync_stat[(sync_status >> 8) & 0x03])
        self.spdif_freq.setText("%d Hz" % (spdif_freq))

    # Hide and disable a control
    def disable_hide(self,widget):
        widget.hide()
        widget.setEnabled(False)

    def initValues(self):

        # print self.hw.servername
        # print self.hw.basepath
        self.inputmatrix = MatrixMixer(self.hw.servername, self.hw.basepath+"/Mixer/InputFaders", self, 0x8000, self.hw.basepath+"/Mixer/InputMutes", self.hw.basepath+"/Mixer/InputInverts", True)
        layout = QtGui.QVBoxLayout()
        scrollarea = QtGui.QScrollArea()
        scrollarea.setWidgetResizable(True)
        scrollarea.setWidget(self.inputmatrix)
        layout.addWidget(scrollarea)
        self.mixer.setLayout(layout)

        self.playbackmatrix = MatrixMixer(self.hw.servername, self.hw.basepath+"/Mixer/PlaybackFaders", self, 0x8000, self.hw.basepath+"/Mixer/PlaybackMutes", self.hw.basepath+"/Mixer/PlaybackInverts", True)
        layout = QtGui.QVBoxLayout()
        scrollarea = QtGui.QScrollArea()
        scrollarea.setWidgetResizable(True)
        scrollarea.setWidget(self.playbackmatrix)
        layout.addWidget(scrollarea)
        self.playbackmixer.setLayout(layout)

        self.outputmatrix = MatrixMixer(self.hw.servername, self.hw.basepath+"/Mixer/OutputFaders", self, 0x8000, self.hw.basepath+"/Mixer/OutputMutes", None, True)
        layout = QtGui.QVBoxLayout()
        scrollarea = QtGui.QScrollArea()
        scrollarea.setWidget(self.outputmatrix)
        scrollarea.setWidgetResizable(True)

        # This is a bit of a hack, but it works to ensure this single-row
        # matrix mixer doesn't fill the entire screen but also doesn't end
        # up with a pointless scrollbar.  The matrix mixer's minimum height
        # is 0 according to minimumHeight(), which is probably the
        # fundamental issue here; however, I've already wasted too much time
        # trying to get this to work so if the hack is effective we'll run
        # with it.
        scrollarea.setMinimumHeight(150)
        layout.addWidget(scrollarea, 0, Qt.AlignTop)
        self.outputmixer.setLayout(layout)

        self.is_streaming = False
        self.last_streaming_state = False

        # Retrieve other device settings as needed and customise the UI
        # based on these options.
        self.model = self.hw.getDiscrete('/Control/Model')
        log.debug("device model identifier: %d" % (self.model))
        self.tco_present = self.hw.getDiscrete('/Control/TCO_present')
        log.debug("device has TCO: %d" % (self.tco_present))
        #self.sample_rate = self.hw.getDiscrete('/Mixer/Info/SampleRate')
        #log.debug("device sample rate: %d" % (self.sample_rate))

        # The Fireface-400 only has 2 phantom-capable channels
        if (self.model == RME_MODEL_FF400):
            self.disable_hide(self.phantom_2)
            self.disable_hide(self.phantom_3)
        else:
            self.phantom_0.setText("Mic 7")
            self.phantom_1.setText("Mic 8")
            self.phantom_2.setText("Mic 9")
            self.phantom_3.setText("Mic 10")

        # Instrument options, input jack selection controls and an ADAT2
        # input are applicable only to the FF800
        if (self.model != RME_MODEL_FF800):
            self.instrument_options_group.setEnabled(False)
            self.input_plug_select_group.setEnabled(False)
            self.sync_ref_adat2.setEnabled(False)
            self.sync_check_adat2_label.setEnabled(False)
            self.sync_check_adat2_status.setEnabled(False)

        for ctrl, info in self.Combos.iteritems():
            if (not(ctrl.isEnabled())):
                continue;
            val = self.hw.getDiscrete(info[0])
            log.debug("combo %s is %d" % (info[0], val));
            ctrl.setCurrentIndex(val);
            QObject.connect(ctrl, SIGNAL('currentIndexChanged(int)'), self.updateCombo)

        if (not(self.tco_present)):
            self.sync_check_tco_label.setEnabled(False)
            self.sync_check_tco_status.setEnabled(False)
            self.sync_ref_tco.setEnabled(False)

        # Only the FF400 has specific channel 3/4 options, input gain
        # controls and switchable phones level
        if (self.model != RME_MODEL_FF400):
            self.disable_hide(self.input_gains_group)
            self.disable_hide(self.channel_3_4_options_group)
            self.phones_level_group.setEnabled(False)

        # Add the "No ADAT-2" item to the bandwidth limit control if the
        # device is not a FF400.  Set the control to reflect the current
        # device setting and connect an update signal.
        if (self.model != RME_MODEL_FF400):
            self.bandwidth_limit.insertItem(1, "No ADAT-2")
        val = self.hw.getDiscrete('/Control/Bandwidth_limit')
        if (self.model==RME_MODEL_FF400 and val>1):
            val = val - 1
        self.bandwidth_limit.setCurrentIndex(val);
        QObject.connect(self.bandwidth_limit, SIGNAL('currentIndexChanged(int)'), self.updateBandwidthLimit)

        # Get current hardware values and connect GUI element signals to 
        # their respective slots
        for ctrl, info in self.PhantomSwitches.iteritems():
            if (not(ctrl.isEnabled())):
                continue
            val = (self.hw.getDiscrete(info[0]) >> info[1]) & 0x01
            log.debug("phantom switch %d is %d" % (info[1], val))
            if val:
                ctrl.setChecked(True)
            else:
                ctrl.setChecked(False)
            QObject.connect(ctrl, SIGNAL('toggled(bool)'), self.updatePhantomSwitch)

        for ctrl, info in self.Switches.iteritems():
            if (not(ctrl.isEnabled())):
                continue
            val = self.hw.getDiscrete(info[0])
            log.debug("switch %s is %d" % (info[0], val))
            if val:
                ctrl.setChecked(True)
            else:
                ctrl.setChecked(False)
            QObject.connect(ctrl, SIGNAL('toggled(bool)'), self.updateSwitch)

        for ctrl, info in self.Radiobuttons.iteritems():
            if (not(ctrl.isEnabled())):
                continue;
            # This is a touch wasteful since it means we retrieve the control
            # value once per radio button rather than once per radio button
            # group.  In time we might introduce radiobutton groupings in the
            # self.* datastructures to avoid this, but for the moment this is
            # easy and it works.
            val = self.hw.getDiscrete(info[0])
            if (val == info[1]):
                val = 1
            else:
                val = 0
            ctrl.setChecked(val)
            log.debug("Radiobutton %s[%d] is %d" % (info[0], info[1], val))
            QObject.connect(ctrl, SIGNAL('toggled(bool)'), self.updateRadiobutton)

        for ctrl, info in self.Checkboxes.iteritems():
            if (not(ctrl.isEnabled())):
                continue;
            # This is a touch wasteful since it means we retrieve the control
            # value once per checkbox button rather than once per checkbox
            # group.  In time we might introduce checkbox groupings in the
            # self.* datastructures to avoid this, but for the moment this is
            # easy and it works.
            val = self.hw.getDiscrete(info[0])
            if (val & info[1]):
                val = 1
            else:
                val = 0
            ctrl.setChecked(val)
            log.debug("Checkbox %s[%d] is %d" % (info[0], info[1], val))
            QObject.connect(ctrl, SIGNAL('toggled(bool)'), self.updateCheckboxes)

        for ctrl, info in self.Gains.iteritems():
            if (not(ctrl.isEnabled())):
                continue
            val = self.hw.getMatrixMixerValue(info[0], 0, info[1])
            log.debug("gain %s[%d] is %d" % (info[0], info[1], val))
            ctrl.setValue(val);
            QObject.connect(ctrl, SIGNAL('valueChanged(int)'), self.updateGain)

        self.updateStreamingState()
        #log.debug("device streaming flag: %d" % (self.is_streaming))

        self.update_timer = QTimer(self)
        QObject.connect(self.update_timer, SIGNAL('timeout()'), self.status_update)
        self.update_timer.start(1000)

# vim: et



#
# Copyright (C) 2005-2008 by Pieter Palmers
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from PyQt4.QtCore import SIGNAL, SLOT, QObject
from PyQt4.QtGui import QWidget
from ffado.config import *

import logging
log = logging.getLogger('phase88')

class Phase88Control(QWidget):
    def __init__(self,parent = None):
        QWidget.__init__(self,parent)
        uicLoad("ffado/mixer/phase88", self)

        self.VolumeControls={
            'master':    ['/Mixer/Feature_Volume_1', self.sldInputMaster], 
            'line12' :   ['/Mixer/Feature_Volume_2', self.sldInput12],
            'line34' :   ['/Mixer/Feature_Volume_3', self.sldInput34],
            'line56' :   ['/Mixer/Feature_Volume_4', self.sldInput56],
            'line78' :   ['/Mixer/Feature_Volume_5', self.sldInput78],
            'spdif' :    ['/Mixer/Feature_Volume_6', self.sldInputSPDIF],
            'waveplay' : ['/Mixer/Feature_Volume_7', self.sldInputWavePlay],
            }

        self.SelectorControls={
            'outassign':    ['/Mixer/Selector_6', self.comboOutAssign], 
            'inassign':     ['/Mixer/Selector_7', self.comboInAssign], 
            'externalsync': ['/Mixer/Selector_8', self.comboExtSync], 
            'syncsource':   ['/Mixer/Selector_9', self.comboSyncSource], 
            'frontback':    ['/Mixer/Selector_10', self.comboFrontBack], 
        }

    def switchFrontState(self,a0):
        self.setSelector('frontback', a0)

    def switchOutAssign(self,a0):
        self.setSelector('outassign', a0)

    def switchWaveInAssign(self,a0):
        self.setSelector('inassign', a0)

    def switchSyncSource(self,a0):
        self.setSelector('syncsource', a0)

    def switchExtSyncSource(self,a0):
        self.setSelector('externalsync', a0)

    def setVolume12(self,a0):
        self.setVolume('line12', a0)

    def setVolume34(self,a0):
        self.setVolume('line34', a0)

    def setVolume56(self,a0):
        self.setVolume('line56', a0)

    def setVolume78(self,a0):
        self.setVolume('line78', a0)

    def setVolumeSPDIF(self,a0):
        self.setVolume('spdif', a0)

    def setVolumeWavePlay(self,a0):
        self.setVolume('waveplay', a0)

    def setVolumeMaster(self,a0):
        self.setVolume('master', a0)

    def setVolume(self,a0,a1):
        name=a0
        vol = -a1
        log.debug("setting %s volume to %d" % (name, vol))
        self.hw.setContignuous(self.VolumeControls[name][0], vol)

    def setSelector(self,a0,a1):
        name=a0
        state = a1
        log.debug("setting %s state to %d" % (name, state))
        self.hw.setDiscrete(self.SelectorControls[name][0], state)

    def initValues(self):
        for name, ctrl in self.VolumeControls.iteritems():
            vol = self.hw.getContignuous(ctrl[0])
            log.debug("%s volume is %d" % (name , vol))
            ctrl[1].setValue(-vol)

        for name, ctrl in self.SelectorControls.iteritems():
            state = self.hw.getDiscrete(ctrl[0])
            log.debug("%s state is %d" % (name , state))
            ctrl[1].setCurrentIndex(state)

# vim: et



#!/usr/bin/python
#
# Copyright (C) 2007-2008 Arnold Krille
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import imp

def pyuic4_action( target, source, env ):
	env.Execute( "pyuic4 " + str( source[0] ) + " > " + str( target[0] ) )
	return 0

def pyuic4_string( target, source, env ):
	return "building '%s' from '%s'" % ( str(target[0]), str( source[0] ) )

def PyQt4Check( context ):
	context.Message( "Checking for pyuic4 (by checking for the python module pyqtconfig) " )
	ret = True
	try:
		imp.find_module( "pyqtconfig" )
	except ImportError:
		ret = False
	context.Result( ret )
	return ret

def generate( env, **kw ):
	env['BUILDERS']['PyUIC4'] = env.Builder( action=pyuic4_action, src_suffix=".ui", single_source=True )
	env['PYUIC4_TESTS'] = { "PyQt4Check" : PyQt4Check }


def exists( env ):
	return 1




#
# Copyright (C) 2005-2008 by Daniel Wagner
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from PyQt4.QtCore import SIGNAL, SLOT, QObject
from PyQt4.QtGui import QWidget
from ffado.config import *
import logging
log = logging.getLogger('panelmanager')

class Ozonic(QWidget):
    def __init__(self,parent = None):
        QWidget.__init__(self,parent)
        uicLoad("ffado/mixer/ozonic", self)

        self.Controls={
            'fader1' :   ['/Mixer/Feature_Volume_1', 1,  self.fader1],
            'fader2' :   ['/Mixer/Feature_Volume_1', 2,  self.fader2],
            'fader3' :   ['/Mixer/Feature_Volume_2', 1,  self.fader3],
            'fader4' :   ['/Mixer/Feature_Volume_2', 2,  self.fader4],
            'fader5' :   ['/Mixer/Feature_Volume_3', 1,  self.fader5],
            'fader6' :   ['/Mixer/Feature_Volume_3', 2,  self.fader6],
            'fader7' :   ['/Mixer/Feature_Volume_4', 1,  self.fader7],
            'fader8' :   ['/Mixer/Feature_Volume_4', 2,  self.fader8],
            'pan1' :   ['/Mixer/Feature_LRBalance_1', 1,  self.pan1],
            'pan2' :   ['/Mixer/Feature_LRBalance_1', 2,  self.pan2],
            'pan3' :   ['/Mixer/Feature_LRBalance_2', 1,  self.pan3],
            'pan4' :   ['/Mixer/Feature_LRBalance_2', 2,  self.pan4],
            'pan5' :   ['/Mixer/Feature_LRBalance_3', 1,  self.pan5],
            'pan6' :   ['/Mixer/Feature_LRBalance_3', 2,  self.pan6],
            'pan7' :   ['/Mixer/Feature_LRBalance_4', 1,  self.pan7],
            'pan8' :   ['/Mixer/Feature_LRBalance_4', 2,  self.pan8],
            }

    def setFader1(self,a0):
        self.setValue('fader1', a0)

    def setFader2(self,a0):
        self.setValue('fader2', a0)

    def setFader3(self,a0):
        self.setValue('fader3', a0)

    def setFader4(self,a0):
        self.setValue('fader4', a0)

    def setFader5(self,a0):
        self.setValue('fader5', a0)

    def setFader6(self,a0):
        self.setValue('fader6', a0)

    def setFader7(self,a0):
        self.setValue('fader7', a0)

    def setFader8(self,a0):
        self.setValue('fader8', a0)

    def setPan1(self,a0):
        self.setValue('pan1', a0)

    def setPan2(self,a0):
        self.setValue('pan2', a0)

    def setPan3(self,a0):
        self.setValue('pan3', a0)

    def setPan4(self,a0):
        self.setValue('pan4', a0)

    def setPan5(self,a0):
        self.setValue('pan5', a0)

    def setPan6(self,a0):
        self.setValue('pan6', a0)

    def setPan7(self,a0):
        self.setValue('pan7', a0)

    def setPan8(self,a0):
        self.setValue('pan8', a0)

    def setValue(self,a0,a1):
        name = a0
        val = a1
        ctrl = self.Controls[name]
        if name[0:3] == 'pan' :
            val = -a1
        log.debug("setting %s to %d" % (name, val))
        self.hw.setContignuous(ctrl[0],  val,  idx = ctrl[1])

    def initValues(self):
        for name, ctrl in self.Controls.iteritems():
            val = self.hw.getContignuous(ctrl[0],  idx = ctrl[1])
            if name[0:3] == 'pan' :
                val = -val
            log.debug("%s value is %d" % (name , val))
            ctrl[2].setValue(val)

# vim: et



#!/usr/bin/python
#
# Copyright (C) 2005-2007 by Pieter Palmers
#               2007-2008 by Arnold Krille
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import sys

import os
import time
import dbus

class ControlInterface:
    def __init__(self, servername, basepath):
        self.basepath=basepath
        self.servername=servername
        self.bus=dbus.SessionBus()

    def setContignuous(self, subpath, v, idx=None):
        try:
            path = self.basepath + subpath
            dev = self.bus.get_object(self.servername, path)
            dev_cont = dbus.Interface(dev, dbus_interface='org.ffado.Control.Element.Continuous')
            if idx == None:
                dev_cont.setValue(v)
            else:
                dev_cont.setValueIdx(idx,v)
        except:
            print "Failed to set Continuous %s on server %s" % (path, self.servername)

    def getContignuous(self, subpath, idx=None):
        try:
            path = self.basepath + subpath
            dev = self.bus.get_object(self.servername, path)
            dev_cont = dbus.Interface(dev, dbus_interface='org.ffado.Control.Element.Continuous')
            if idx == None:
                return dev_cont.getValue()
            else:
                return dev_cont.getValueIdx(idx)
        except:
            print "Failed to get Continuous %s on server %s" % (path, self.servername)
            return 0

    def setDiscrete(self, subpath, v):
        try:
            path = self.basepath + subpath
            dev = self.bus.get_object(self.servername, path)
            dev_cont = dbus.Interface(dev, dbus_interface='org.ffado.Control.Element.Discrete')
            dev_cont.setValue(v)
        except:
            print "Failed to set Discrete %s on server %s" % (path, self.servername)

    def getDiscrete(self, subpath):
        try:
            path = self.basepath + subpath
            dev = self.bus.get_object(self.servername, path)
            dev_cont = dbus.Interface(dev, dbus_interface='org.ffado.Control.Element.Discrete')
            return dev_cont.getValue()
        except:
            print "Failed to get Discrete %s on server %s" % (path, self.servername)
            return 0

    def setRegister(self, subpath, address, value):
        try:
            path = self.basepath + subpath
            dev = self.bus.get_object(self.servername, path)
            dev_cont = dbus.Interface(dev, dbus_interface='org.ffado.Control.Element.Register')
            dev_cont.setValue(address, value)
        except:
            print "Failed to set Register %s on server %s" % (path, self.servername)

    def getRegister(self, subpath, address):
        try:
            path = self.basepath + subpath
            dev = self.bus.get_object(self.servername, path)
            dev_cont = dbus.Interface(dev, dbus_interface='org.ffado.Control.Element.Register')
            return dev_cont.getValue(address)
        except:
            print "Failed to get Register %s on server %s" % (path, self.servername)
            return 0

    def setText(self, subpath, v):
        try:
            path = self.basepath + subpath
            dev = self.bus.get_object(self.servername, path)
            dev_cont = dbus.Interface(dev, dbus_interface='org.ffado.Control.Element.Text')
            dev_cont.setValue(v)
        except:
            print "Failed to set Text %s on server %s" % (path, self.servername)

    def getText(self, subpath):
        try:
            path = self.basepath + subpath
            dev = self.bus.get_object(self.servername, path)
            dev_cont = dbus.Interface(dev, dbus_interface='org.ffado.Control.Element.Text')
            return dev_cont.getValue()
        except:
            print "Failed to get Text %s on server %s" % (path, self.servername)
            return 0

    def setMatrixMixerValue(self, subpath, row, col, v):
        try:
            path = self.basepath + subpath
            dev = self.bus.get_object(self.servername, path)
            dev_cont = dbus.Interface(dev, dbus_interface='org.ffado.Control.Element.MatrixMixer')
            dev_cont.setValue(row, col, v)
        except:
            print "Failed to set MatrixMixer %s on server %s" % (path, self.servername)

    def getMatrixMixerValue(self, subpath, row, col):
        try:
            path = self.basepath + subpath
            dev = self.bus.get_object(self.servername, path)
            dev_cont = dbus.Interface(dev, dbus_interface='org.ffado.Control.Element.MatrixMixer')
            return dev_cont.getValue(row, col)
        except:
            print "Failed to get MatrixMixer %s on server %s" % (path, self.servername)
            return 0

    def enumSelect(self, subpath, v):
        try:
            path = self.basepath + subpath
            dev = self.bus.get_object(self.servername, path)
            dev_cont = dbus.Interface(dev, dbus_interface='org.ffado.Control.Element.Enum')
            dev_cont.select(v)
        except:
            print "Failed to select %s on server %s" % (path, self.servername)

    def enumSelected(self, subpath):
        try:
            path = self.basepath + subpath
            dev = self.bus.get_object(self.servername, path)
            dev_cont = dbus.Interface(dev, dbus_interface='org.ffado.Control.Element.Enum')
            return dev_cont.selected()
        except:
            print "Failed to get selected enum %s on server %s" % (path, self.servername)
            return 0

    def enumGetLabel(self, subpath, v):
        try:
            path = self.basepath + subpath
            dev = self.bus.get_object(self.servername, path)
            dev_cont = dbus.Interface(dev, dbus_interface='org.ffado.Control.Element.Enum')
            return dev_cont.getEnumLabel(v)
        except:
            print "Failed to get enum label %s on server %s" % (path, self.servername)
            return 0

    def enumCount(self, subpath):
        try:
            path = self.basepath + subpath
            dev = self.bus.get_object(self.servername, path)
            dev_cont = dbus.Interface(dev, dbus_interface='org.ffado.Control.Element.Enum')
            return dev_cont.count()
        except:
            print "Failed to get enum count %s on server %s" % (path, self.servername)
            return 0

class DeviceManagerInterface:
    def __init__(self, servername, basepath):
        self.basepath=basepath + '/DeviceManager'
        self.servername=servername
        self.bus=dbus.SessionBus()
        self.dev = self.bus.get_object(self.servername, self.basepath)
        self.iface = dbus.Interface(self.dev, dbus_interface='org.ffado.Control.Element.Container')
        # signal reception does not work yet since we need a mainloop for that
        # and qt3 doesn't provide one for python/dbus
        #try:
            #self.dev.connect_to_signal("Updated", self.updateSignal, \
                                       #dbus_interface="org.ffado.Control.Element.Container", arg0=self)
        #except dbus.DBusException:
            #traceback.print_exc()

    #def updateSignal(self):
        #print ("Received update signal")

    def getNbDevices(self):
        return self.iface.getNbElements()
    def getDeviceName(self, idx):
        return self.iface.getElementName(idx)


class ConfigRomInterface:
    def __init__(self, servername, devicepath):
        self.basepath=devicepath + '/ConfigRom'
        self.servername=servername
        self.bus=dbus.SessionBus()
        self.dev = self.bus.get_object(self.servername, self.basepath)
        self.iface = dbus.Interface(self.dev, dbus_interface='org.ffado.Control.Element.ConfigRomX')
    def getGUID(self):
        return self.iface.getGUID()
    def getVendorName(self):
        return self.iface.getVendorName()
    def getModelName(self):
        return self.iface.getModelName()
    def getVendorId(self):
        return self.iface.getVendorId()
    def getModelId(self):
        return self.iface.getModelId()
    def getUnitVersion(self):
        return self.iface.getUnitVersion()

class ClockSelectInterface:
    def __init__(self, servername, devicepath):
        self.basepath=devicepath + '/Generic/ClockSelect'
        self.servername=servername
        self.bus=dbus.SessionBus()
        self.dev = self.bus.get_object(self.servername, self.basepath)
        self.iface = dbus.Interface(self.dev, dbus_interface='org.ffado.Control.Element.AttributeEnum')
    def count(self):
        return self.iface.count()
    def select(self, idx):
        return self.iface.select(idx)
    def selected(self):
        return self.iface.selected()
    def getEnumLabel(self, idx):
        return self.iface.getEnumLabel(idx)
    def attributeCount(self):
        return self.iface.attributeCount()
    def getAttributeValue(self, idx):
        return self.iface.getAttributeValue(idx)
    def getAttributeName(self, idx):
        return self.iface.getAttributeName(idx)

class TextInterface:
    def __init__(self, servername, basepath):
        self.basepath=basepath
        self.servername=servername
        self.bus=dbus.SessionBus()
        self.dev = self.bus.get_object( self.servername, self.basepath )
        self.iface = dbus.Interface( self.dev, dbus_interface="org.ffado.Control.Element.Text" )

    def text(self):
        return self.iface.getValue()

    def setText(self,text):
        self.iface.setValue(text)

if __name__ == "__main__":

    print """
===========================================================
FFADO DBUS TEST TOOL
===========================================================
"""
    
    server='org.ffado.Control'
    basepath='/org/ffado/Control'

    repeat = 1
    while repeat > 0:
        try:
            devmgr = DeviceManagerInterface(server, basepath)
            nbDevices = devmgr.getNbDevices()
            repeat -= 1
        except dbus.DBusException, ex:
            print "\n"
            print "==========================================================="
            print "ERROR: Could not communicate with the FFADO DBus service..."
            print "==========================================================="
            print "\n"
            sys.exit( -1 )

    if nbDevices == 0:
        print "No supported device found..."
        sys.exit( -1 )

    idx = 0

    path=devmgr.getDeviceName(idx)
    print "Found device %d: %s" % (idx, path)

    cfgrom = ConfigRomInterface(server, basepath+'/DeviceManager/'+path)
    vendorId = cfgrom.getVendorId()
    modelId = cfgrom.getModelId()
    unitVersion = cfgrom.getUnitVersion()
    GUID = cfgrom.getGUID()
    vendorName = cfgrom.getVendorName()
    modelName = cfgrom.getModelName()
    print " Found (%s, %X, %X) %s %s" % (str(GUID), vendorId, modelId, vendorName, modelName)

    # create a control objects
    hw = ControlInterface(server, basepath+'/DeviceManager/'+path)
    clockselect = ClockSelectInterface( server, basepath+"/DeviceManager/"+path )
    nickname = TextInterface( server, basepath+"/DeviceManager/"+path+"/Generic/Nickname" )

    import time
    register = range(60)
    # do whatever you have to do
    for i in range(60):
        for regid in range(60):
            newval = hw.getRegister("/Register", regid)
            oldval = register[regid]
            if newval != oldval:
                print "%04d: from %8d | %08X to %8d | %08X" % (regid, oldval, oldval, newval, newval)
                register[regid] = newval
            time.sleep(0.2)
        print "--- %d ---" % i
        #time.sleep(1)

    #regid=eval(sys.argv[1])
    #val=eval(sys.argv[2])
    #print hw.getRegister("/Register", regid)
    #hw.setRegister("/Register", regid, val)
    


#!/usr/bin/python
#
# Copyright (C) 2007-2008 Arnold Krille
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Taken from http://www.scons.org/wiki/UsingPkgConfig
# and heavily modified
#

import subprocess

#
# Checks for pkg-config
#
def CheckForPKGConfig( context, version='0.0.0' ):
	context.Message( "Checking for pkg-config (at least version %s)... " % version )
	ret = context.TryAction( "pkg-config --atleast-pkgconfig-version=%s" %version )[0]
	context.Result( ret )
	return ret

#
# Checks for the given package with an optional version-requirement
#
# The flags (which can be imported into the environment by env.MergeFlags(...)
# are exported as env['NAME_FLAGS'] where name is built by removing all +,-,.
# and upper-casing.
#
def CheckForPKG( context, name, version="" ):
	name2 = name.replace("+","").replace(".","").replace("-","")

	if version == "":
		context.Message( "Checking for %s... \t" % name )
		ret = context.TryAction( "pkg-config --exists '%s'" % name )[0]
	else:
		context.Message( "Checking for %s (%s or higher)... \t" % (name,version) )
		ret = context.TryAction( "pkg-config --atleast-version=%s '%s'" % (version,name) )[0]

	if ret:
		context.env['%s_FLAGS' % name2.upper()] = context.env.ParseFlags("!pkg-config --cflags --libs %s" % name)

	context.Result( ret )
	return ret

#
# Checks for the existance of the package and returns the packages flags.
#
# This should allow caching of the flags so that pkg-config is called only once.
#
def GetPKGFlags( context, name, version="" ):
	import os

	if version == "":
		context.Message( "Checking for %s... \t" % name )
		ret = context.TryAction( "pkg-config --exists '%s'" % name )[0]
	else:
		context.Message( "Checking for %s (%s or higher)... \t" % (name,version) )
		ret = context.TryAction( "pkg-config --atleast-version=%s '%s'" % (version,name) )[0]

	if not ret:
		context.Result( ret )
		return ret

	out = subprocess.Popen(['pkg-config', '--cflags', '--libs', name], stdout=subprocess.PIPE)
	ret = out.stdout.read()

	context.Result( True )
	return ret

#
# Checks for the existance of the package and returns the value of the specified variable.
#
def GetPKGVariable( context, name, variable ):
	import os

	context.Message( "Checking for variable %s in package %s... \t" % (variable,name) )

	ret = context.TryAction( "pkg-config --exists '%s'" % name )[0]
	if not ret:
		context.Result( ret )
		return ret

	out = subprocess.Popen(['pkg-config', '--variable=%s' % variable, name], stdout=subprocess.PIPE)
	ret = out.stdout.read()

	context.Result( True )
	return ret

def generate( env, **kw ):
	env['PKGCONFIG_TESTS' ] = { 'CheckForPKGConfig' : CheckForPKGConfig, 'CheckForPKG' : CheckForPKG, 'GetPKGFlags' : GetPKGFlags, 'GetPKGVariable' : GetPKGVariable }

def exists( env ):
	return 1





#
# Copyright (C) 2009 by Arnold Krille
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from PyQt4 import QtGui, QtCore
import dbus, math

import logging
log = logging.getLogger("crossbarrouter")

class VuMeter(QtGui.QFrame):
    def __init__(self, interface, output, input=None, parent=None):
        QtGui.QFrame.__init__(self, parent)
        self.setLineWidth(1)
        self.setFrameStyle(QtGui.QFrame.Panel|QtGui.QFrame.Sunken)
        self.setMinimumSize(20, 20)

        self.level = 0

        self.interface = interface

        self.output = output

    def updateLevel(self, value):
        self.level = value
        self.update()

    def paintEvent(self, event):
        p = QtGui.QPainter(self)
        value = self.level/4096
        r = self.rect()
        r.setHeight(r.height() * math.sqrt(value))
        r.moveBottom(self.rect().height())
        p.fillRect(r, self.palette().highlight())

class OutputSwitcher(QtGui.QFrame):
    """
The name is a bit misleading. This widget selectes sources for a specified
destination.

In mixer-usage this widget is at the top of the input-channel. Because the input
of the mixer is an available output from the routers point.
"""
    def __init__(self, interface, outname, parent):
        QtGui.QFrame.__init__(self, parent)
        self.interface = interface
        self.outname = outname
        self.lastin = ""

        self.setLineWidth(1)
        self.setFrameStyle(QtGui.QFrame.Sunken|QtGui.QFrame.Panel)

        self.layout = QtGui.QGridLayout(self)
        self.setLayout(self.layout)

        self.lbl = QtGui.QLabel(self.outname, self)
        self.lbl.setToolTip("The name of the destination that is to be controlled here.")
        self.layout.addWidget(self.lbl, 0, 0)

        self.vu = VuMeter(self.interface, outname, parent=self)
        self.layout.addWidget(self.vu, 0, 1)

        sources = self.interface.getSourceNames()

        self.combo = QtGui.QComboBox(self)
        self.combo.setToolTip("<qt>Select the source for this destination.<br>Each destination can only receive sound from one source at a time. But one source can send sound to multiple destinations.</qt>")
        self.layout.addWidget(self.combo, 1, 0, 1, 2)
        self.combo.addItem("Disconnected")
        self.combo.addItems(sources)
        src = self.interface.getSourceForDestination(self.outname)
        if src != "":
            self.combo.setCurrentIndex(self.combo.findText(src))
        else:
            self.combo.setCurrentIndex(0)
        self.connect(self.combo, QtCore.SIGNAL("activated(QString)"), self.comboCurrentChanged)


    def peakValue(self, value):
        self.vu.updateLevel(value)
        pass

    def comboCurrentChanged(self, inname):
        #log.debug("comboCurrentChanged( %s )" % inname)
        if inname == self.lastin:
            return
        if self.lastin != "":
            self.interface.setConnectionState(self.lastin, self.outname, False)

        if inname != "Disconnected":
            if self.interface.setConnectionState(str(inname), self.outname, True):
                self.lastin = str(inname)
            else:
                log.warning(" Failed to connect %s to %s" % (inname, self.outname))
        else:
            self.lastin = ""


class CrossbarRouter(QtGui.QWidget):
    def __init__(self, servername, basepath, parent=None):
        QtGui.QWidget.__init__(self, parent);
        self.bus = dbus.SessionBus()
        self.dev = self.bus.get_object(servername, basepath)
        self.interface = dbus.Interface(self.dev, dbus_interface="org.ffado.Control.Element.CrossbarRouter")

        self.settings = QtCore.QSettings(self)

        destinations = self.interface.getDestinationNames()
        self.outgroups = []
        for ch in destinations:
            tmp = str(ch).split(":")[0]
            if not tmp in self.outgroups:
                self.outgroups.append(tmp)

        self.biglayout = QtGui.QVBoxLayout(self)
        self.setLayout(self.biglayout)

        self.toplayout = QtGui.QHBoxLayout()
        self.biglayout.addLayout(self.toplayout)

        self.vubtn = QtGui.QPushButton("Switch peak meters", self)
        self.vubtn.setCheckable(True)
        self.connect(self.vubtn, QtCore.SIGNAL("toggled(bool)"), self.runVu)
        self.toplayout.addWidget(self.vubtn)

        self.layout = QtGui.QGridLayout()
        self.biglayout.addLayout(self.layout)

        self.switchers = {}
        for out in destinations:
            btn = OutputSwitcher(self.interface, out, self)
            self.layout.addWidget(btn, int(out.split(":")[-1]) + 1, self.outgroups.index(out.split(":")[0]))
            self.switchers[out] = btn

        self.timer = QtCore.QTimer(self)
        self.timer.setInterval(200)
        self.connect(self.timer, QtCore.SIGNAL("timeout()"), self.updateLevels)

        self.vubtn.setChecked(self.settings.value("crossbarrouter/runvu", False).toBool())

    def __del__(self):
        print "CrossbarRouter.__del__()"
        self.settings.setValue("crossbarrouter/runvu", self.vubtn.isChecked())

    def runVu(self, run=True):
        #log.debug("CrossbarRouter.runVu( %i )" % run)
        if run:
            self.timer.start()
        else:
            self.timer.stop()
            for sw in self.switchers:
                self.switchers[sw].peakValue(0)

    def updateLevels(self):
        #log.debug("CrossbarRouter.updateLevels()")
        peakvalues = self.interface.getPeakValues()
        #log.debug("Got %i peaks" % len(peakvalues))
        for peak in peakvalues:
            #log.debug("peak = [%s,%s]" % (str(peak[0]),str(peak[1])))
            if peak[0] >= 0:
                self.switchers[peak[0]].peakValue(peak[1])

#
# vim: sw=4 ts=4 et



#
# Copyright (C) 2008 by Arnold Krille
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from PyQt4.QtCore import QObject, pyqtSignature
from PyQt4.QtGui import QWidget, QMessageBox
from ffado.config import *

import logging
log = logging.getLogger('global')

class GlobalMixer(QWidget):
    def __init__(self, parent, name=None):
        QWidget.__init__(self, parent)
        uicLoad("ffado/mixer/globalmixer", self)
        self.setName(name)

    def setName(self,name):
        if name is not None:
            self.lblName.setText(name)
            self.lblName.show()
        else:
            self.lblName.hide()

    @pyqtSignature("int")
    def on_clocksource_activated( self, clock ):
        #log.debug("updateClockSource( " + str(clock) + " )")
        if self.clockselect.canChangeValue():
            self.clockselect.select( clock )
        else:
            msg = QMessageBox()
            msg.question( msg, "Error", \
                "<qt>Clock source change not permitted. Is streaming active?</qt>", \
                QMessageBox.Ok )
            self.clocksource.setEnabled(False)
            return

        selected = self.clockselect.selected()
        if selected != clock:
            clockname = self.clockselect.getEnumLabel( clock )
            msg = QMessageBox()
            msg.question( msg, "Failed to select clock source", \
                "<qt>Could not select %s as clock source.</qt>" % clockname, \
                QMessageBox.Ok )
            self.clocksource.setCurrentIndex( selected )

    @pyqtSignature("int")
    def on_samplerate_activated( self, sr ):
        log.debug("on_samplerate_activated( " + str(sr) + " )")
        if self.samplerateselect.canChangeValue():
            self.samplerateselect.select( sr )
        else:
            msg = QMessageBox()
            msg.question( msg, "Error", \
                "<qt>Sample rate change not permitted. Is streaming active?</qt>", \
                QMessageBox.Ok )
            self.samplerate.setEnabled(False)
            return

        selected = self.samplerateselect.selected()
        if selected != sr:
            srname = self.samplerateselect.getEnumLabel( sr )
            msg = QMessageBox()
            msg.question( msg, "Failed to select sample rate", \
                "<qt>Could not select %s as samplerate.</qt>" % srname, \
                QMessageBox.Ok )
            self.samplerate.setCurrentIndex( selected )

    @pyqtSignature("")
    def on_txtNickname_returnPressed( self ):
        if self.nickname.canChangeValue():
            asciiData = self.txtNickname.text().toAscii()
            self.nickname.setText( asciiData.data() )
        else:
            self.txtNickname.setText( self.nickname.text() )

    def initValues( self ):
        #print "GlobalMixer::initValues()"
        nb_clocks = self.clockselect.count()
        for i in range( nb_clocks ):
            self.clocksource.insertItem( nb_clocks, self.clockselect.getEnumLabel( i ) )
        self.clocksource.setCurrentIndex( self.clockselect.selected() )

        nb_rates = self.samplerateselect.count()
        for i in range( nb_rates ):
            self.samplerate.insertItem( nb_rates, self.samplerateselect.getEnumLabel( i ) )
        self.samplerate.setCurrentIndex( self.samplerateselect.selected() )

        self.txtNickname.setText( self.nickname.text() )

        self.samplerate.setEnabled(self.samplerateselect.canChangeValue())
        self.clocksource.setEnabled(self.clockselect.canChangeValue())
        if self.nickname.canChangeValue():
            self.txtNickname.setEnabled(True)
        else:
            self.txtNickname.setEnabled(False)

        self.streaming_status = self.streamingstatus.selected()

    def polledUpdate(self):
        self.samplerate.setEnabled(self.samplerateselect.canChangeValue())
        self.clocksource.setEnabled(self.clockselect.canChangeValue())
        self.txtNickname.setEnabled(self.nickname.canChangeValue())
        ss = self.streamingstatus.selected()
        ss_txt = self.streamingstatus.getEnumLabel(ss)
        if ss_txt == 'Idle':
            self.chkStreamIn.setChecked(False)
            self.chkStreamOut.setChecked(False)
        elif ss_txt == 'Sending':
            self.chkStreamIn.setChecked(False)
            self.chkStreamOut.setChecked(True)
        elif ss_txt == 'Receiving':
            self.chkStreamIn.setChecked(True)
            self.chkStreamOut.setChecked(False)
        elif ss_txt == 'Both':
            self.chkStreamIn.setChecked(True)
            self.chkStreamOut.setChecked(True)

        if (ss!=self.streaming_status and ss_txt!='Idle'):
            self.samplerate.setCurrentIndex( self.samplerateselect.selected() )
        self.streaming_status = ss

# vim: et



#
# Copyright (C) 2009-2010 by Arnold Krille
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from PyQt4 import QtGui, QtCore, Qt
import dbus

from ffado.widgets.matrixmixer import MatrixMixer
from ffado.widgets.crossbarrouter import *

from ffado.config import *

class Generic_Dice_EAP(QtGui.QWidget):
    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        self.layout = QtGui.QGridLayout(self)
        self.setLayout(self.layout)
        self.tabs = QtGui.QTabWidget(self)
        self.layout.addWidget(self.tabs)

    def buildMixer(self):
        #print self.hw
        #print self.hw.getText("/Generic/Nickname")
        self.matrix = MatrixMixer(self.hw.servername, self.hw.basepath+"/EAP/MatrixMixer", self)
        scrollarea = QtGui.QScrollArea(self.tabs)
        scrollarea.setWidgetResizable(True)
        scrollarea.setWidget(self.matrix)
        self.tabs.addTab(scrollarea, "Matrix Mixer")

        self.router = CrossbarRouter(self.hw.servername, self.hw.basepath+"/EAP/Router", self)
        scrollarea = QtGui.QScrollArea(self.tabs)
        scrollarea.setWidgetResizable(True)
        scrollarea.setWidget(self.router)
        self.tabs.addTab(scrollarea, "Crossbar Router")


    #def getDisplayTitle(self):
    #    return "Saffire PRO40/PRO24 Mixer"

#
# vim: et ts=4 sw=4



#!/usr/bin/python
#
# Copyright (C) 2005-2008 by Pieter Palmers
#               2007-2009 by Arnold Krille
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os

from ffado.config import *

import subprocess

from PyQt4.QtCore import SIGNAL, SLOT, QObject, QTimer, Qt
from PyQt4.QtGui import *

from ffado.dbus_util import *

from ffado.panelmanager import PanelManager

from ffado.logginghandler import *

"""Just a small helper to ask the retry-question without a blocking messagebox"""
class StartDialog(QWidget):
    def __init__(self, parent):
        QWidget.__init__(self, parent)
        self.setObjectName("Restart Dialog")
        self.label = QLabel("<qt>Somehow the connection to the dbus-service of FFADO couldn't be established.<p>\nShall we take another try?</qt>",self)
        self.button = QPushButton("Retry", self)
        self.layout = QGridLayout(self)
        self.layout.setContentsMargins( 50, 10, 50, 10 )
        self.layout.addWidget(self.label, 0, 0, Qt.AlignHCenter|Qt.AlignBottom)
        self.layout.addWidget(self.button, 1, 0, Qt.AlignHCenter|Qt.AlignTop)

class FFADOWindow(QMainWindow):
    def __init__(self, parent):
        QMainWindow.__init__(self, parent)

        self.textlogger = QTextLogger(self)
        dock = QDockWidget("Log Messages",self)
        dock.setWidget(self.textlogger.textedit)
        logging.getLogger('').addHandler(self.textlogger)
        self.addDockWidget(Qt.BottomDockWidgetArea, dock)

        self.statuslogger = QStatusLogger(self, self.statusBar(), 20)
        logging.getLogger('').addHandler(self.statuslogger)

        self.manager = PanelManager(self)
        self.connect(self.manager, SIGNAL("connectionLost"), self.connectToDBUS)

        filemenu = self.menuBar().addMenu("File")
        quitaction = QAction("Quit", self)
        quitaction.setShortcut(self.tr("Ctrl+q"))
        self.connect(quitaction, SIGNAL("triggered()"), self, SLOT("close()"))
        filemenu.addAction(quitaction)

        editmenu = self.menuBar().addMenu("Edit")
        self.updateaction = QAction("Update Mixer Panels", self)
        self.updateaction.setEnabled(False)
        self.connect(self.updateaction, SIGNAL("triggered()"), self.manager.updatePanels)
        editmenu.addAction(self.updateaction)
        refreshaction = QAction("Refresh Current Panels", self)
        self.connect(refreshaction, SIGNAL("triggered()"), self.manager.refreshPanels)
        editmenu.addAction(refreshaction)

        helpmenu = self.menuBar().addMenu( "Help" )
        aboutaction = QAction( "About FFADO", self )
        self.connect( aboutaction, SIGNAL( "triggered()" ), self.aboutFFADO )
        helpmenu.addAction( aboutaction )
        aboutqtaction = QAction( "About Qt", self )
        self.connect( aboutqtaction, SIGNAL( "triggered()" ), qApp, SLOT( "aboutQt()" ) )
        helpmenu.addAction( aboutqtaction )

        log.info( "Starting up" )

        QTimer.singleShot( 1, self.tryStartDBUSServer )

    def __del__(self):
        log.info("__del__")
        del self.manager
        log.info("__del__ finished")

    def closeEvent(self, event):
        log.info("closeEvent()")
        event.accept()

    def connectToDBUS(self):
        log.info("connectToDBUS")
        try:
            self.setupDeviceManager()
        except dbus.DBusException, ex:
            log.error("Could not communicate with the FFADO DBus service...")
            if not hasattr(self,"retry"):
                self.retry = StartDialog(self)
                self.connect(self.retry.button, SIGNAL("clicked()"), self.tryStartDBUSServer)
            if hasattr(self, "retry"):
                self.manager.setParent(None)
                self.setCentralWidget(self.retry)
                self.retry.setEnabled(True)

    def tryStartDBUSServer(self):
        try:
            self.setupDeviceManager()
        except dbus.DBusException, ex:
            if hasattr(self, "retry"):
                self.retry.setEnabled(False)
            subprocess.Popen(['ffado-dbus-server', '-v3']).pid
            QTimer.singleShot(5000, self.connectToDBUS)

    def setupDeviceManager(self):
        devmgr = DeviceManagerInterface(FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH)
        self.manager.setManager(devmgr)
        if hasattr(self, "retry"):
            self.retry.setParent(None)
        self.setCentralWidget(self.manager)
        self.updateaction.setEnabled(True)

    def aboutFFADO(self):
        QMessageBox.about( self, "About FFADO", """
<h1>ffado.org</h1>

<p>FFADO is the new approach to have firewire audio on linux.</p>

<p>&copy; 2006-2008 by the FFADO developers<br />ffado is licensed under the GPLv3, for the full license text see <a href="http://www.gnu.org/licenses/">www.gnu.org/licenses</a> or the LICENSE.* files shipped with ffado.</p>

<p>FFADO developers are:<ul>
<li>Pieter Palmers
<li>Daniel Wagner
<li>Jonathan Woithe
<li>Arnold Krille
</ul>
""" )


def get_lock(process_name):
    import socket
    import sys

    global lock_socket
    lock_socket = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
    try:
        lock_socket.bind('\0' + process_name)
        # Lock acquired
    except socket.error:
        print 'ffado-mixer instance is already running'
        sys.exit()


def ffadomain(args):
    #set up logging
    import logging
    logging.basicConfig( datefmt="%H:%M:%S", format="%(asctime)s %(name)-16s %(levelname)-8s %(message)s" )

    if DEBUG:
        debug_level = logging.DEBUG
    else:
        debug_level = logging.INFO

    get_lock('ffado-mixer')

    # main loggers:
    logging.getLogger('main').setLevel(debug_level)
    logging.getLogger('dbus').setLevel(debug_level)
    logging.getLogger('registration').setLevel(debug_level)
    logging.getLogger('panelmanager').setLevel(debug_level)
    logging.getLogger('configparser').setLevel(logging.INFO)

    # widgets:
    logging.getLogger('matrixmixer').setLevel(debug_level)
    logging.getLogger('crossbarrouter').setLevel(debug_level)

    # mixers:
    logging.getLogger('audiofire').setLevel(debug_level)
    logging.getLogger('bridgeco').setLevel(debug_level)
    logging.getLogger('edirolfa101').setLevel(debug_level)
    logging.getLogger('edirolfa66').setLevel(debug_level)
    logging.getLogger('motu').setLevel(debug_level)
    logging.getLogger('rme').setLevel(debug_level)
    logging.getLogger('phase24').setLevel(debug_level)
    logging.getLogger('phase88').setLevel(debug_level)
    logging.getLogger('quatafire').setLevel(debug_level)
    logging.getLogger('saffirebase').setLevel(debug_level)
    logging.getLogger('saffire').setLevel(debug_level)
    logging.getLogger('saffirepro').setLevel(debug_level)

    logging.getLogger('global').setLevel(debug_level)

    log = logging.getLogger('main')

    app = QApplication(args)
    app.setWindowIcon( QIcon( SHAREDIR + "/icons/hi64-apps-ffado.png" ) )

    app.setOrganizationName("FFADO")
    app.setOrganizationDomain("ffado.org")
    app.setApplicationName("ffado-mixer")

    mainwindow = FFADOWindow(None)


    # rock & roll
    mainwindow.show()
    return app.exec_()

if __name__ == "__main__":
    import sys
    sys.exit(ffadomain(sys.argv))

#
# vim: ts=4 sw=4 et



#!/usr/bin/python
#
# Copyright (C) 2007-2008 Arnold Krille
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Taken from http://www.scons.org/wiki/ReplacementBuilder
#

from string import Template
import os

def replace_action(target, source, env):
	open( str(target[0]), 'w' ).write( Template( open( str(source[0]), 'r' ).read() ).safe_substitute( env ) )
	os.chmod( str(target[0]), os.stat( str(source[0]) ).st_mode )
	return 0

def replace_string(target, source, env):
	return "building '%s' from '%s'" % ( str(target[0]), str(source[0]) )

def generate(env, **kw):
	action = env.Action( replace_action, replace_string )
	env['BUILDERS']['ScanReplace'] = env.Builder( action=action, src_suffix='.in', single_source=True )

def exists(env):
	return 1




#
# Copyright (C) 2010 by Arnold Krille <arnold@arnoldarts.de>
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from PyQt4 import QtGui, QtCore, Qt
import dbus

from ffado.config import *

class Presonus_Experimental(QtGui.QWidget):
    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        uicLoad('ffado/mixer/presonus_experimental', self)

    def buildMixer(self):
        #print self.hw
        #print 'Servername = "%s"\nBasename = "%s"' % (self.hw.servername, self.hw.basepath)
        bus = self.hw.bus
        self.ifaces = list()
        for feature in ['Volume_1', 'Volume_2', 'Volume_3', 'Volume_4', 'LRBalance_1', 'LRBalance_2', 'LRBalance_3', 'LRBalance_4']:
            dev = bus.get_object(self.hw.servername, self.hw.basepath+'/Mixer/Feature_%s' % feature)
            #print dev
            iface = dbus.Interface(dev, dbus_interface='org.ffado.Control.Element.Continuous')
            self.ifaces.append(iface)
            #print iface
            #print iface.getValue(), iface.getMinimum(), iface.getMaximum()
            obj = self.findChild(QtGui.QWidget, feature)
            #print obj
            obj.setMinimum(iface.getMinimum())
            obj.setMaximum(iface.getMaximum())
            obj.setValue(iface.getValue())
            self.connect(obj, QtCore.SIGNAL("valueChanged(int)"), iface.setValue)

#
# vim: et ts=4 sw=4



#
# Copyright (C) 2009-2010 by Arnold Krille
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from PyQt4 import QtGui, QtCore, Qt
import dbus

from ffado.widgets.matrixmixer import MatrixMixer
from ffado.widgets.crossbarrouter import *
from ffado.mixer.generic_dice_eap import *

from ffado.config import *

class BooleanControl:
    def __init__(self, hw, path):
        self.iface = dbus.Interface(
                hw.bus.get_object(hw.servername, path),
                dbus_interface="org.ffado.Control.Element.Boolean")
        self.value = self.iface.selected()

    def selected(self):
        return self.value

    def select(self, n):
        if self.iface.select(n):
            self.value = n
            return True
        return False

class DiscreteControl:
    def __init__(self, hw, path):
        self.iface = dbus.Interface(
                hw.bus.get_object(hw.servername, path),
                dbus_interface="org.ffado.Control.Element.Discrete")
        self.value = self.iface.getValue()

    def getvalue(self):
        return self.value

    def setvalue(self, v):
        if v != self.value:
            self.iface.setValue(v)
            self.value = v

class MonitoringModel(QtCore.QAbstractTableModel):
    def __init__(self, hw, parent):
        QtCore.QAbstractTableModel.__init__(self, parent)

        self.hw = hw

        self.mutestates = ("Global", "True", "False")

        self.globaldims = []
        self.globalmutes = []
        self.perchannelmutes = []
        self.globalvolumes = []
        self.perchannelvolumes = []
        for i in range(10):
            self.globaldims.append(BooleanControl(self.hw, self.hw.basepath + ("/EAP/Monitoring/GlobalDim/AffectsCh%i" % i)))
            self.globalmutes.append(BooleanControl(self.hw, self.hw.basepath + ("/EAP/Monitoring/GlobalMute/AffectsCh%i" % i)))
            self.perchannelmutes.append(BooleanControl(self.hw, self.hw.basepath + ("/EAP/Monitoring/PerChannel/Mute%i" % i)))
            self.globalvolumes.append(BooleanControl(self.hw, self.hw.basepath + ("/EAP/Monitoring/GlobalVolume/AffectsCh%i" % i)))
            self.perchannelvolumes.append(DiscreteControl(self.hw, self.hw.basepath + ("/EAP/Monitoring/PerChannel/Volume%i" % i)))

    def rowCount(self, parent):
        return 3

    def columnCount(self, parent):
        return 10

    def headerData(self, section, orientation, role=Qt.Qt.DisplayRole):
        if role != Qt.Qt.DisplayRole:
            #print "headerData will return nothing"
            return None
        #print "headerData() Will return a label"
        if orientation == Qt.Qt.Horizontal:
            return "Ch. %i" % section
        if orientation == Qt.Qt.Vertical:
            return ("Mute", "Dim", "Volume")[section]
        return None

    def data(self, index, role):
        if not role in (Qt.Qt.DisplayRole, Qt.Qt.EditRole):
            return None
        row = index.row()
        col = index.column()
        if role == Qt.Qt.DisplayRole:
            if row is 0:
                if self.perchannelmutes[col].selected():
                    return self.mutestates[1]
                else:
                    if self.globalmutes[col].selected():
                        return self.mutestates[0]
                return self.mutestates[2]
            if row is 1:
                if self.globaldims[col].selected():
                    return "Enabled"
                else:
                    return "Disabled"
            if row is 2:
                if self.globalvolumes[col].selected():
                    return "Global"
                else:
                    return self.perchannelvolumes[col].getvalue()
        if role == Qt.Qt.EditRole:
            if row is 0:
                return QtCore.QStringList(self.mutestates)
            if row is 1:
                return QtCore.QStringList(("Enabled","Disabled"))
            if row is 2:
                if self.globalvolumes[col].selected():
                    return 1
                return self.perchannelvolumes[col].getvalue()
        return "%i,%i" % (row,col)

    def setData(self, index, value, role=Qt.Qt.EditRole):
        col = index.column()
        row = index.row()
        if row == 0:
            if value == "Global":
                self.globalmutes[col].select(True)
                self.perchannelmutes[col].select(False)
            else:
                self.globalmutes[col].select(False)
                if value == "True": value = True
                if value == "False": value = False
                self.perchannelmutes[col].select(value)
            return True
        if row == 1:
            if value == "Enabled": value = True
            if value == "Disabled": value = False
            return self.globaldims[col].select(value)
        if row == 2:
            #print "setData() value=%s" % value.toString()
            v = int(value.toString())
            #print " integer is %i" % v
            if v > 0:
                return self.globalvolumes[col].select(True)
            else:
                self.globalvolumes[col].select(False)
                self.perchannelvolumes[col].setvalue(v)
        return False

    def flags(self, index):
        ret = QtCore.QAbstractTableModel.flags(self, index)
        if index.row() in (0,1,2):
            ret |= Qt.Qt.ItemIsEditable
        return ret



class MonitoringDelegate(QtGui.QItemDelegate):
    def __init__(self, parent):
        QtGui.QItemDelegate.__init__(self, parent)

    def createEditor(self, parent, option, index):
        if index.data(Qt.Qt.EditRole).type() == QtCore.QVariant.StringList:
            combo = QtGui.QComboBox(parent)
            self.connect(combo, QtCore.SIGNAL("activated(int)"), self.currentChanged)
            return combo
        else:
            return QtGui.QItemDelegate.createEditor(self, parent, option, index)

    def setEditorData(self, editor, index):
        if isinstance(editor, QtGui.QComboBox):
            list = index.data(Qt.Qt.EditRole).toStringList()
            editor.addItems(list)
            editor.setCurrentIndex(list.indexOf(index.data(Qt.Qt.DisplayRole).toString()))
        else:
            QtGui.QItemDelegate.setEditorData(self, editor, index)

    def setModelData(self, editor, model, index):
        if isinstance(editor, QtGui.QComboBox):
            model.setData(index, editor.currentText(), Qt.Qt.EditRole)
        else:
            QtGui.QItemDelegate.setModelData(self, editor, model, index)

    def currentChanged(self):
        #print "currentChanged() sender=%s" % (str(self.sender()))
        editor = self.sender()
        self.emit(QtCore.SIGNAL("commitData(QWidget*)"), editor)
        self.emit(QtCore.SIGNAL("closeEditor(QWidget*)"), editor)

class Saffire_Dice(Generic_Dice_EAP):
    def __init__(self, parent=None):
        Generic_Dice_EAP.__init__(self, parent)

    def buildMixer(self):
        #print self.hw
        #print self.hw.getText("/Generic/Nickname")
        Generic_Dice_EAP.buildMixer(self)

        model = MonitoringModel(self.hw, self)

        widget = QtGui.QWidget()
        uicLoad("ffado/mixer/saffire_dice_monitoring.ui", widget)
        widget.monitoringView.setModel(model)
        widget.monitoringView.setItemDelegate(MonitoringDelegate(self))
        self.tabs.addTab(widget, "Monitoring")

        self.muteInterface = BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/GlobalMute/State")
        widget.btnMute.setChecked(self.muteInterface.selected())
        self.connect(widget.btnMute, QtCore.SIGNAL("toggled(bool)"), self.muteToggle)
        self.dimInterface = BooleanControl(self.hw, self.hw.basepath+"/EAP/Monitoring/GlobalDim/State")
        widget.btnDim.setChecked(self.dimInterface.selected())
        self.connect(widget.btnDim, QtCore.SIGNAL("toggled(bool)"), self.dimToggle)

        self.dimLevelInterface = DiscreteControl(self.hw, self.hw.basepath+"/EAP/Monitoring/GlobalDim/Level")
        widget.dimLevel.setValue(self.dimLevelInterface.getvalue())
        self.connect(widget.dimLevel, QtCore.SIGNAL("valueChanged(int)"), self.dimLevelChanged)
        self.volumeInterface = DiscreteControl(self.hw, self.hw.basepath+"/EAP/Monitoring/GlobalVolume/Level")
        widget.volumeKnob.setValue(self.volumeInterface.getvalue())
        self.connect(widget.volumeKnob, QtCore.SIGNAL("valueChanged(int)"), self.volumeChanged)

        if self.configrom.getModelName() == "SAFFIRE_PRO_24" or self.configrom.getModelName() == "SAFFIRE_PRO_24DSP":
            widget.stacked.setCurrentWidget(widget.pagePro24)
            self.ch1inst = BooleanControl(self.hw, self.hw.basepath + "/EAP/Ch1LineInst")
            widget.chkInst1.setChecked(self.ch1inst.selected())
            self.connect(widget.chkInst1, QtCore.SIGNAL("toggled(bool)"), self.ch1inst.select)
            self.ch2inst = BooleanControl(self.hw, self.hw.basepath + "/EAP/Ch2LineInst")
            widget.chkInst2.setChecked(self.ch2inst.selected())
            self.connect(widget.chkInst2, QtCore.SIGNAL("toggled(bool)"), self.ch2inst.select)
            self.ch3level = BooleanControl(self.hw, self.hw.basepath + "/EAP/Ch3Level")
            widget.chkLevel3.setChecked(self.ch3level.selected())
            self.connect(widget.chkLevel3, QtCore.SIGNAL("toggled(bool)"), self.ch3level.select)
            self.ch4level = BooleanControl(self.hw, self.hw.basepath + "/EAP/Ch4Level")
            widget.chkLevel4.setChecked(self.ch4level.selected())
            self.connect(widget.chkLevel4, QtCore.SIGNAL("toggled(bool)"), self.ch4level.select)
            #widget.chkSpdif.deleteLater()
            #widget.btnPad.deleteLater()
        elif self.configrom.getModelName() == "SAFFIRE_PRO_40":
            widget.stacked.setCurrentWidget(widget.pagePro40)
            #widget.chkInst1.deleteLater()
            #widget.chkInst2.deleteLater()
            #widget.chkLevel3.deleteLater()
            #widget.chkLevel4.deleteLater()
        else:
            self.stacked.deleteLater()


    def muteToggle(self, mute):
        self.muteInterface.select(mute)
    def dimToggle(self, mute):
        self.dimInterface.select(mute)

    def dimLevelChanged(self, value):
        self.dimLevelInterface.setvalue(value)
    def volumeChanged(self, value):
        self.volumeInterface.setvalue(value)

    def getDisplayTitle(self):
        return "Saffire PRO40/PRO24 Mixer"

#
# vim: et ts=4 sw=4



#!/usr/bin/python
#

#
# Copyright (C) 2008 Pieter Palmers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import commands
import re

LISTIRQINFO_VERSION="0.3"

def sortedDictValues(adict):
    items = adict.items()
    items.sort()
    return [value for key, value in items]

class IRQ:
    def __init__(self):
        self.number = None
        self.scheduling_class = None
        self.scheduling_priority = None
        self.process_id = None
        self.drivers = []
        self.cpu_counts = []
    def __str__(self):
        s = " IRQ %4s: PID: %5s, count: %18s, Sched %4s (priority %4s), drivers: %s" % \
            (self.number, self.process_id, self.cpu_counts,
             self.scheduling_class, self.scheduling_priority,
             self.drivers)
        return s

class SoftIRQ:
    def __init__(self):
        self.name = None
        self.fullname = None
        self.scheduling_class = None
        self.scheduling_priority = None
        self.process_id = None
        self.cpu_counts = []
    def __str__(self):
        s = " SoftIRQ %12s: PID %6s, Sched %4s (priority %4s), name: %s" % \
            (self.name, self.process_id ,self.scheduling_class, self.scheduling_priority, self.fullname)
        return s

class IRQInfo:
    def __init__(self):
        self.softIRQs = {}
        self.IRQs = {}

    def load(self):
        # get PID info
        (exitstatus, outtext) = commands.getstatusoutput('ps -eLo pid,cmd,class,rtprio | grep IRQ')
        
        rawstr = r"""([0-9]+) +\[IRQ-([0-9]+)\] +([A-Z]{2}) +([-0-9]+)"""
        compile_obj = re.compile(rawstr)
        
        IRQs = {}
        for line in outtext.splitlines():
            match_obj = compile_obj.search(line)
            if match_obj:
                irq = IRQ()
                irq.process_id = int(match_obj.group(1))
                irq.number = int(match_obj.group(2))
                irq.scheduling_class = match_obj.group(3)
                if match_obj.group(4) != '-':
                    irq.scheduling_priority = int(match_obj.group(4))
                else:
                    irq.scheduling_priority = None
                IRQs[irq.number] = irq
        
        (exitstatus, outtext) = commands.getstatusoutput('ps -eLo pid,cmd,class,rtprio | grep softirq')
        
        rawstr = r"""([0-9]+) +\[softirq-(.*)\] +([A-Z]+) +([-0-9]+)"""
        compile_obj = re.compile(rawstr)
        
        softIRQs = {}
        for line in outtext.splitlines():
            match_obj = compile_obj.search(line)
            if match_obj:
                irq = SoftIRQ()
                irq.process_id = int(match_obj.group(1))
                irq.name = "%s-%s" % (match_obj.group(2),irq.process_id)
                irq.fullname = "softirq-%s" % match_obj.group(2)
                irq.scheduling_class = match_obj.group(3)
                if match_obj.group(4) != '-':
                    irq.scheduling_priority = int(match_obj.group(4))
                else:
                    irq.scheduling_priority = None
                softIRQs[irq.name] = irq
        
        # get irq info
        (exitstatus, outtext) = commands.getstatusoutput('cat /proc/interrupts')
        lines = outtext.splitlines()
        nb_cpus = len(lines[0].split())
        str0 = "([0-9]+): +";
        str_cpu = "([0-9]+) +"
        str1="([\w\-]+) +([\w\-, :]+)"
        
        rawstr = str0;
        for i in range(nb_cpus):
            rawstr += str_cpu
        rawstr += str1
        compile_obj = re.compile(rawstr)
        
        for line in outtext.splitlines():
            match_obj = compile_obj.search(line)
            if match_obj:
                irq_number = int(match_obj.group(1))
                if not irq_number in IRQs.keys():
                    IRQs[irq_number] = IRQ()
                    IRQs[irq_number].number = irq_number
                
                irq = IRQs[irq_number]
                cpu_counts = []
                for i in range(nb_cpus):
                    cpu_counts.append(int(match_obj.group(i + 2)))
                irq.cpu_counts = cpu_counts
                irq.type = match_obj.group(nb_cpus + 2)
                drivers = match_obj.group(nb_cpus + 3).split(',')
                for driver in drivers:
                    irq.drivers.append(driver.strip())
        
                IRQs[irq.number] = irq

        self.softIRQs = softIRQs
        self.IRQs = IRQs

    def __str__(self):
        s  = ""
        s += "Hardware Interrupts:\n"
        s += "--------------------\n"

        for irq in sortedDictValues(self.IRQs):
            s += str(irq) + "\n"

        s += "\n"
        s += "Software Interrupts:\n"
        s += "--------------------\n"
        for irq in sortedDictValues(self.softIRQs):
            s += str(irq) + "\n"

        return s

if __name__== '__main__':

    print ""
    print "Interrupt list utility " + LISTIRQINFO_VERSION
    print "=========================="
    print "(C) 2008 Pieter Palmers"
    print ""
    
    info = IRQInfo()

    info.load()
    print str(info)
    
    print ""



#!/usr/bin/python
#
# Copyright (C) 2005-2009 by Pieter Palmers
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import dbus

GUID = "00130e0401400045"

servername = "org.ffado.Control"
path = "/org/ffado/Control/DeviceManager/%s/EAP/" % GUID
bus = dbus.SessionBus()

router = dbus.Interface(bus.get_object(servername, path+"Router"),
                        dbus_interface='org.ffado.Control.Element.CrossbarRouter')
mixer = dbus.Interface(bus.get_object(servername, path+"MatrixMixer"),
                        dbus_interface='org.ffado.Control.Element.MatrixMixer')

# mute the mixer
#nbinputs = mixer.getRowCount()
#nboutputs = mixer.getColCount()

#print nbinputs
#print nboutputs

#for i in range(nbinputs):
    #for o in range(nboutputs):
        #mixer.setValue(i, o, 0)

## reconfigure the router
#def reroute(src, dsts):
    #srcidx = router.getSourceIndex(src)
    #for dst in dsts:
        #print dst
        #dstidx = router.getDestinationIndex(dst)
    
        #sourceidx = 0
        ## disconnect
        #while sourceidx >= 0:
            #sourceidx = router.getSourceForDestination(dstidx)
            #router.setConnectionState(sourceidx, dstidx, 0)
    
        ## connect
        #if router.canConnect(srcidx, dstidx):
            #print router.getConnectionState(srcidx, dstidx)
            #print router.setConnectionState(srcidx, dstidx, 1)

#reroute("InS1:02", ["InS0:00", "InS1:00", "InS1:02", "InS1:04", "InS1:06"])
#reroute("InS1:03", ["InS0:01", "InS1:01", "InS1:03", "InS1:05", "InS1:07"])

tst = router.getConnectionMap()

nb_sources = router.getNbSources()
nb_destinations = router.getNbDestinations()

source_names = router.getSourceNames()
destination_names = router.getDestinationNames()


srcnames_formatted = ["%10s" % s for s in source_names]
s = ""
for i in range(10):
    s += "           |"
    for src in range(nb_sources):
        name = srcnames_formatted[src]
        char = name[i]
        s += " %s" % char
    s += "\n"

s += "-----------+"
for src in range(nb_sources):
    s += "--"
s += "\n"

for dst in range(nb_destinations):
    s += "%10s |" % destination_names[dst]
    for src in range(nb_sources):
        idx = dst * nb_sources + src
        s += " %d" % tst[idx]
    s += "\n"

print s



#
# Copyright (C) 2005-2008 by Pieter Palmers
#               2007-2008 by Arnold Krille
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from ffado.config import * #FFADO_VERSION, FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH

from PyQt4.QtGui import QFrame, QWidget, QTabWidget, QVBoxLayout, QMainWindow, QIcon, QAction, qApp, QStyleOptionTabWidgetFrame
from PyQt4.QtCore import QTimer

from ffado.dbus_util import *
from ffado.registration import *

from ffado.configuration import *

from ffado.mixer.globalmixer import GlobalMixer
from ffado.mixer.dummy import Dummy

import time

import logging
log = logging.getLogger('panelmanager')

use_generic = False
try:
    from mixer_generic import *
    log.info("The generic mixer is found, seems to be a developer using ffadomixer...")
except ImportError:
    pass
else:
    use_generic = True

# pseudo-guid
GUID_GENERIC_MIXER = 0


class HLine( QFrame ):
    def __init__( self, parent ):
        QFrame.__init__( self, parent )
        self.setFrameShape( QFrame.HLine )
        self.setLineWidth( 2 )
        self.setMinimumHeight( 10 )

class PanelManagerStatus(QWidget):
    def __init__(self, parent):
        QWidget.__init__(self,parent)
        uicLoad("ffado/panelmanagerstatus", self)

class OwnTabWidget(QTabWidget):
    def __init__(self,parent):
        QTabWidget.__init__(self,parent)

    def tabInserted(self,index):
        self.checkTabBar()

    def tabRemoved(self,index):
        self.checkTabBar()

    def checkTabBar(self):
        if self.count()<2:
            self.tabBar().hide()
        else:
            self.tabBar().show()

class PanelManager(QWidget):
    def __init__(self, parent, devmgr=None):
        QWidget.__init__(self, parent)
        self.setObjectName("PanelManager")

        # maps a device GUID to a QT panel
        self.panels = {}

        # a layout for ourselves
        self.layout = QVBoxLayout(self)

        # the tabs
        self.tabs = OwnTabWidget(self)
        self.tabs.hide()
        self.layout.addWidget(self.tabs)

        # a dialog that is shown during update
        self.status = PanelManagerStatus(self)
        self.layout.addWidget(self.status)
        self.status.show()

        self.devices = DeviceList( SYSTEM_CONFIG_FILE )
        self.devices.updateFromFile( USER_CONFIG_FILE )

        if devmgr is not None:
            self.setManager(devmgr)

    def __del__(self):
        print("PanelManager.__del__()")
        self.polltimer.stop()

    def setManager(self,devmgr):
        self.devmgr = devmgr
        self.devmgr.registerPreUpdateCallback(self.devlistPreUpdate)
        self.devmgr.registerPostUpdateCallback(self.devlistPostUpdate)
        self.devmgr.registerUpdateCallback(self.devlistUpdate)
        self.devmgr.registerDestroyedCallback(self.devmgrDestroyed)
        # create a timer to poll the panels
        self.polltimer = QTimer()
        self.connect( self.polltimer, SIGNAL('timeout()'), self.pollPanels )
        self.polltimer.start( POLL_SLEEP_TIME_MSEC )

        # create a timer to initialize the panel after the main form is shown
        # since initialization can take a while
        QTimer.singleShot( POLL_SLEEP_TIME_MSEC, self.updatePanels )

        # live check timer
        self.alivetimer = QTimer()
        QObject.connect( self.alivetimer, SIGNAL('timeout()'), self.commCheck )
        self.alivetimer.start( 2000 )

    def count(self):
        return self.tabs.count()

    def pollPanels(self):
        #log.debug("PanelManager::pollPanels()")
        # only when not modifying the tabs
        try:
            if self.tabs.isEnabled():
                for guid in self.panels.keys():
                    w = self.panels[guid]
                    for child in w.children():
                        #log.debug("poll child %s,%s" % (guid,child))
                        if 'polledUpdate' in dir(child):
                            child.polledUpdate()
        except:
            log.error("error in pollPanels")
            self.commCheck()

    def devlistPreUpdate(self):
        log.debug("devlistPreUpdate")
        self.tabs.setEnabled(False)
        self.tabs.hide()
        self.status.lblMessage.setText("Bus reconfiguration in progress, please wait...")
        self.status.show()
        #self.statusBar().showMessage("bus reconfiguration in progress...", 5000)

    def devlistPostUpdate(self):
        log.debug("devlistPostUpdate")
        # this can fail if multiple busresets happen in fast succession
        ntries = 10
        while ntries > 0:
            try:
                self.updatePanels()
                return
            except:
                log.debug("devlistPostUpdate failed (%d)" % ntries)
                for guid in self.panels.keys():
                    w = self.panels[guid]
                    del self.panels[guid] # remove from the list
                    idx = self.tabs.indexOf(w)
                    self.tabs.removeTab(idx)
                    del w # GC might also take care of that

                ntries = ntries - 1
                time.sleep(2) # sleep a few seconds

        log.debug("devlistPostUpdate failed completely")
        self.tabs.setEnabled(False)
        self.tabs.hide()
        self.status.lblMessage.setText("Error while reconfiguring. Please restart ffadomixer.")
        self.status.show()


    def devlistUpdate(self):
        log.debug("devlistUpdate")

    def devmgrDestroyed(self):
        log.debug("devmgrDestroyed")
        self.alivetimer.stop()
        self.tabs.setEnabled(False)
        self.tabs.hide()
        self.status.lblMessage.setText("DBUS server was shut down, please restart it and restart ffadomixer...")
        self.status.show()

    def commCheck(self):
        try:
            nbDevices = self.devmgr.getNbDevices()
        except:
            log.error("The communication with ffado-dbus-server was lost.")
            self.tabs.setEnabled(False)
            self.polltimer.stop()
            self.alivetimer.stop()
            keys = self.panels.keys()
            for panel in keys:
                w = self.panels[panel]
                del self.panels[panel]
                w.deleteLater()
            self.emit(SIGNAL("connectionLost"))

    def removePanel(self, guid):
        print "Removing widget for device" + guid
        w = self.panels[guid]
        del self.panels[guid] # remove from the list
        idx = self.tabs.indexOf(w)
        self.tabs.removeTab(idx)
        w.deleteLater()

    def addPanel(self, idx):
        path = self.devmgr.getDeviceName(idx)
        log.debug("Adding device %d: %s" % (idx, path))

        cfgrom = ConfigRomInterface(FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH+'/DeviceManager/'+path)
        vendorId = cfgrom.getVendorId()
        modelId = cfgrom.getModelId()
        unitVersion = cfgrom.getUnitVersion()
        guid = cfgrom.getGUID()
        vendorName = cfgrom.getVendorName()
        modelName = cfgrom.getModelName()
        log.debug(" Found (%s, %X, %X) %s %s" % (str(guid), vendorId, modelId, vendorName, modelName))

        # check whether this has already been registered at ffado.org
        reg = ffado_registration(FFADO_VERSION, int(guid, 16),
                                     vendorId, modelId,
                                     vendorName, modelName)
        reg.check_for_registration()

        # The MOTU devices use unitVersion to differentiate models.  For the
        # moment though we don't need to know precisely which model we're
        # using.
        if vendorId == 0x1f2:
            modelId = 0x00000000

        # The RME devices use unitVersion to differentiate models. 
        # Therefore in the configuration file we use the config file's
        # modelid field to store the unit version.  As a result we must
        # override the modelId with the unit version here so the correct
        # configuration file entry (and hense mixer widget) is identified.
        if vendorId == 0xa35:
            modelId = unitVersion;

        dev = self.devices.getDeviceById( vendorId, modelId )

        w = QWidget( )
        l = QVBoxLayout( w )

        # create a control object
        hw = ControlInterface(FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH+'/DeviceManager/'+path)
        clockselect = ClockSelectInterface( FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH+"/DeviceManager/"+path )
        samplerateselect = SamplerateSelectInterface( FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH+"/DeviceManager/"+path )
        streamingstatus = StreamingStatusInterface( FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH+"/DeviceManager/"+path )
        nickname = TextInterface( FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH+"/DeviceManager/"+path+"/Generic/Nickname" )

        #
        # Generic elements for all mixers follow here:
        #
        globalmixer = GlobalMixer( w )
        globalmixer.configrom = cfgrom
        globalmixer.clockselect = clockselect
        globalmixer.samplerateselect = samplerateselect
        globalmixer.streamingstatus = streamingstatus
        globalmixer.nickname = nickname
        globalmixer.hw = hw
        globalmixer.initValues()
        l.addWidget( globalmixer, 1 )

        #
        # Line to separate
        #
        l.addWidget( HLine( w ) )

        #
        # Specific (or dummy) mixer widgets get loaded in the following
        #
        if 'mixer' in dev and dev['mixer'] != None:
            mixerapp = dev['mixer']
            exec( """
import ffado.mixer.%s
mixerwidget = ffado.mixer.%s.%s( w )
""" % (mixerapp.lower(), mixerapp.lower(), mixerapp) )
        else:
            mixerwidget = Dummy( w )
            mixerapp = modelName+" (Dummy)"

        #
        # The same for all mixers
        #
        l.addWidget( mixerwidget, 10 )
        mixerwidget.configrom = cfgrom
        mixerwidget.clockselect = clockselect
        mixerwidget.samplerateselect = samplerateselect
        mixerwidget.streamingstatus = streamingstatus
        mixerwidget.nickname = nickname
        mixerwidget.hw = hw
        if 'buildMixer' in dir(mixerwidget):
            mixerwidget.buildMixer()
        if 'initValues' in dir(mixerwidget):
            mixerwidget.initValues()
        if 'getDisplayTitle' in dir(mixerwidget):
            title = mixerwidget.getDisplayTitle()
        else:
            title = mixerapp

        globalmixer.setName(title)
        self.tabs.addTab( w, title )
        self.panels[guid] = w

    def displayPanels(self):
        # if there is no panel, add the no-device message
        # else make sure it is not present
        if self.count() == 0:
            self.tabs.hide()
            self.tabs.setEnabled(False)
            self.status.lblMessage.setText("No supported device found.")
            self.status.show()
            #self.statusBar().showMessage("No supported device found.", 5000)
        else:
            self.tabs.show()
            self.tabs.setEnabled(True)
            self.status.hide()
            #self.statusBar().showMessage("Configured the mixer for %i devices." % self.tabs.count())
            if use_generic:
                #
                # Show the generic (development) mixer if it is available
                #
                w = GenericMixer( devmgr.bus, FFADO_DBUS_SERVER, mw )
                self.tabs.addTab( w, "Generic Mixer" )
                self.panels[GUID_GENERIC_MIXER] = w
    
    def updatePanels(self):
        log.debug("PanelManager::updatePanels()")
        nbDevices = self.devmgr.getNbDevices()
        #self.statusBar().showMessage("Reconfiguring the mixer panels...")

        # list of panels present
        guids_with_tabs = self.panels.keys()

        # build list of guids on the bus now
        guids_present = []
        guid_indexes = {}
        for idx in range(nbDevices):
            path = self.devmgr.getDeviceName(idx)
            cfgrom = ConfigRomInterface(FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH+'/DeviceManager/'+path)
            guid = cfgrom.getGUID()
            guids_present.append(guid)
            guid_indexes[guid] = idx

        # figure out what to remove
        # the special panel (generic)
        # that has (pseudo-)GUID 0
        # is also automatically removed
        to_remove = []
        for guid in guids_with_tabs:
            if not guid in guids_present:
                to_remove.append(guid)
                log.debug("going to remove %s" % str(guid))
            else:
                log.debug("going to keep %s" % str(guid))

        # figure out what to add
        to_add = []
        for guid in guids_present:
            if not guid in guids_with_tabs:
                to_add.append(guid)
                log.debug("going to add %s" % str(guid))

        # update the widget
        for guid in to_remove:
            self.removePanel(guid)

        for guid in to_add:
            # retrieve the device manager index
            idx = guid_indexes[guid]
            self.addPanel(idx)

        self.displayPanels()

    def refreshPanels(self):
        log.debug("PanelManager::refreshPanels()")
        nbDevices = self.devmgr.getNbDevices()
        #self.statusBar().showMessage("Reconfiguring the mixer panels...")

        # list of panels present
        guids_with_tabs = self.panels.keys()

        # build list of guids on the bus now
        guid_indexes = {}
        for idx in range(nbDevices):
            path = self.devmgr.getDeviceName(idx)
            cfgrom = ConfigRomInterface(FFADO_DBUS_SERVER, FFADO_DBUS_BASEPATH+'/DeviceManager/'+path)
            guid = cfgrom.getGUID()
            guid_indexes[guid] = idx

        # remove/create the widget
        for guid in guids_with_tabs:
            self.removePanel(guid)
            idx = guid_indexes[guid]
            self.addPanel(idx)

        self.displayPanels()

# vim: et



#!/usr/bin/python
#
# Copyright (C) 2007-2008 Arnold Krille
# Copyright (C) 2007-2008 Pieter Palmers
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# xml translator
#

def dbusxx_xml2cpp_adaptor_action( target, source, env ):
	env.Execute( "dbusxx-xml2cpp %s --adaptor=%s" % ( source[0], target[0] ) )
	return 0

def dbusxx_xml2cpp_proxy_action( target, source, env ):
	env.Execute( "dbusxx-xml2cpp %s --proxy=%s" % ( source[0], target[0] ) )
	return 0

def generate( env, **kw ):
	env['BUILDERS']['Xml2Cpp_Adaptor'] = env.Builder(action = dbusxx_xml2cpp_adaptor_action,
		suffix = '.h', src_suffix = '.xml')
	env['BUILDERS']['Xml2Cpp_Proxy'] = env.Builder(action = dbusxx_xml2cpp_proxy_action,
		suffix = '.h', src_suffix = '.xml', single_source=True )

def exists( env ):
	return 1




#!/usr/bin/python
#

#
# Copyright (C) 2008 Pieter Palmers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# A script to install FFADO in a 'sandbox', i.e. without changing the 
# system installation.
#
import os

FFADOSBI_VERSION = '0.1'
LATEST_FFADO_RELEASE_URL = 'http://www.ffado.org/files/libffado-2.0-beta7.tar.gz'
LATEST_FFADO_RELEASE_UNPACK_DIR = 'libffado-2.0-beta7'
LATEST_JACK1_RELEASE_URL = 'http://jackaudio.org/downloads/jack-audio-connection-kit-0.109.2.tar.gz'
LATEST_JACK1_RELEASE_UNPACK_DIR = 'jack-audio-connection-kit-0.109.2'

def ask_for_dir(descr, suggestion):
    ret_dir = None
    while True:
        ret_dir = raw_input("Please specify a %s directory [%s]: " % (descr, suggestion))
        if ret_dir == "":
            ret_dir = suggestion

        if not os.path.exists(ret_dir):
            try:
                os.makedirs(ret_dir)
            except:
                yesno = raw_input("Could not create the %s directory. Try again? [yes/no] " % descr)
                if yesno == "" or yesno[0] != 'y':
                    return None
                else:
                    continue
            break
        else:
            yesno = raw_input("WARNING: the %s directory at %s already exists. Do you want to overwrite it? [yes/no] " % (descr, ret_dir))
            if yesno == "" or yesno[0] != 'y':
                yesno = raw_input("Specify new %s directory? [yes/no] " % descr)
                if yesno == "" or yesno[0] != 'y':
                    return None
                else:
                    continue
            else:
                yesno = raw_input("WARNING: about to remove the old %s directory at %s. Proceed? [yes/no] " % (descr, ret_dir))
                if yesno == "" or yesno[0] != 'y':
                    yesno = raw_input("Specify new %s directory? [yes/no] " % descr)
                    if yesno == "" or yesno[0] != 'y':
                        return None
                    else:
                        continue
                else:
                    os.system('rm -Rf "%s"' % ret_dir)
                    os.makedirs(ret_dir)
                    break
    return ret_dir

def fetch_source(build_dir, source_descriptor, target):
    logfile = "%s/%s.log" % (build_dir, target)
    os.system('echo "" > %s' % logfile)

    if source_descriptor[1] == 'svn':
        print " Checking out SVN repository: %s" % source_descriptor[2]
        cwd = os.getcwd()
        os.chdir(build_dir)
        retval = os.system('svn co "%s" "%s" >> %s' % (source_descriptor[2], target, logfile))
        os.chdir(cwd)
        if retval:
            print "  Failed to checkout the SVN repository. Inspect %s for details. (is subversion installed?)" % logfile
            return False
        return True
    elif source_descriptor[1] == 'tar.gz':
        print " Downloading tarball: %s" % source_descriptor[2]
        import urllib
        tmp_file = '%s/tmp.tar.gz' % build_dir
        try:
            urllib.urlretrieve(source_descriptor[2], tmp_file)
        except:
            print " Could not retrieve source tarball."
            return False
        cwd = os.getcwd()
        os.chdir(build_dir)
        print " extracting tarball..."
        retval = os.system('tar -zxf "%s" > %s' % (tmp_file, logfile))
        if retval:
            print "  Failed to extract the source tarball. Inspect %s for details." % logfile
            os.chdir(cwd)
            return False
        if source_descriptor[3]:
            retval = os.system('mv "%s" "%s"' % (source_descriptor[3], target))
            if retval:
                print "  Failed to move the extracted tarball"
                os.chdir(cwd)
                return False
        os.chdir(cwd)
        return True
    else:
        print "bad source type"
        return False


welcome_msg = """
FFADO sandbox install utility """ + FFADOSBI_VERSION + """
=================================
(C) 2008 Pieter Palmers

This utility will automatically build and install a sandbox
environment that can be used to evaluate FFADO. The required
FFADO and/or jack packages are downloaded.

NOTE: The build dependencies are not automatically installed.
Please consult http://subversion.ffado.org/wiki/Dependencies
for more information. The subversion tool has to be installed.

NOTE: This tool assumes that an internet connection is available
while it runs. Once everything is built, no connection is
required.

You can exit the tool at any time using CTRL-C.

"""

print welcome_msg

# get the paths to be used
if 'HOME' in os.environ.keys():
    suggestion_sandbox = "%s/ffadosandbox" % os.environ['HOME']
else:
    suggestion_sandbox = "/home/myuser/ffadosandbox"

sandbox_dir_msg = """
SANDBOX DIRECTORY
=================

The sandbox directory is the directory where all built files are 
installed into. It should be writable by your user, and will
contain the evaluation binaries if this tool is successful. If
it doesn't exist, it will be created.

Once you are finished with evaluating FFADO you can simply remove
this directory and things are as if nothing happened.

Suggestion: %s

NOTE: if you specify a system directory here, the tool will install
system-wide (if run as root). This is not recommended, but can be
useful for automated installs. Uninstall will be a lot harder though.
""" % suggestion_sandbox

if 'HOME' in os.environ.keys():
    suggestion_build = "%s/ffadobuild" % os.environ['HOME']
else:
    suggestion_build = "/home/myuser/ffadobuild"

build_dir_msg = """
BUILD DIRECTORY
===============

The build directory is where all temporary files are stored while
building FFADO and jack. It should be writable by your user. If
it doesn't exist, it will be created.

The build directory can be removed as soon as this tool has finished.
It is not automatically removed.

Suggestion: %s
""" % suggestion_build

print sandbox_dir_msg
sandbox_dir = ask_for_dir('sandbox', suggestion_sandbox)
if sandbox_dir == None:
    print "Cannot proceed without valid sandbox directory."
    exit(-1)
print(" using %s as sandbox directory" % sandbox_dir)


print build_dir_msg
build_dir = ask_for_dir('build', suggestion_build)
if build_dir == None:
    print "Cannot proceed without valid build directory."
    exit(-1)
print(" using %s as build directory" % build_dir)

# figure out what version of FFADO to build
# Note: format:
# ['title', type=svn or tar.gz, url, (optional) dir that contains target]

ffado_versions = {}
ffado_versions[0] = ['SVN trunk', 'svn', 'http://subversion.ffado.org/ffado/trunk/libffado', None]
ffado_versions[1] = ['SVN libffado-2.0 (recommended)', 'svn', 'http://subversion.ffado.org/ffado/branches/libffado-2.0', None]
ffado_versions[2] = ['latest release', 'tar.gz', LATEST_FFADO_RELEASE_URL, LATEST_FFADO_RELEASE_UNPACK_DIR]

ffado_versions_msg = """
Available FFADO versions:
"""
for key in sorted(ffado_versions.keys()):
    ffado_versions_msg += " %2s: %s\n" % (key, ffado_versions[key][0])

print ffado_versions_msg
while True:
    ffado_version = raw_input("Please select a FFADO version: ")
    try:
        ffado_version_int = eval(ffado_version)
        if ffado_version_int not in [0, 1, 2]:
            raise
    except:
        yesno = raw_input("Invalid FFADO version specified. Try again? [yes/no] ")
        if yesno == "" or yesno[0] != 'y':
            print "Cannot proceed without valid FFADO version."
            exit(-1)
        else:
            continue
    break

use_ffado_version = ffado_versions[ffado_version_int]

# figure out what version of jack to build
jack_versions = {}
jack_versions[0] = ['jack1 SVN trunk', 'svn', 'http://subversion.jackaudio.org/jack/trunk/jack', None]
# note: latest release is no good.
#jack_versions[1] = ['jack1 latest release', 'tar.gz', LATEST_JACK1_RELEASE_URL, LATEST_JACK_RELEASE_UNPACK_DIR]

jack_versions_msg = """
Available jack versions:
"""
for key in sorted(jack_versions.keys()):
    jack_versions_msg += " %2s: %s\n" % (key, jack_versions[key][0])

print jack_versions_msg
while True:
    jack_version = raw_input("Please select a jack version: ")
    try:
        jack_version_int = eval(jack_version)
        if jack_version_int not in [0, 1, 2]:
            raise
    except:
        yesno = raw_input("Invalid jack version specified. Try again? [yes/no] ")
        if yesno == "" or yesno[0] != 'y':
            print "Cannot proceed without valid jack version."
            exit(-1)
        else:
            continue
    break

use_jack_version = jack_versions[jack_version_int]

# print a summary
print """
SUMMARY
=======
Sandbox directory : %s
Build directory   : %s
FFADO version     : %s
Jack version      : %s

""" % (sandbox_dir, build_dir, use_ffado_version[0], use_jack_version[0])

# get the ffado source
print "Fetching FFADO source..."

if not fetch_source(build_dir, use_ffado_version, 'libffado'):
    print "Could not fetch FFADO source"
    exit(-1)
print " Successfully fetched FFADO source"

print "Fetching jack source..."

if not fetch_source(build_dir, use_jack_version, 'jack'):
    print "Could not fetch jack source"
    exit(-1)
print " Successfully fetched jack source"

cwd = os.getcwd()

ffado_log = "%s/ffadobuild.log" % build_dir
ffado_scons_options = "-j2" # TODO: interactive config of the build
os.system('echo "" > %s' % ffado_log) 

# configure FFADO
os.chdir("%s/libffado/" % build_dir)
print "Building FFADO..."
print " Compiling..."
retval = os.system('scons PREFIX="%s" %s >> %s' % (sandbox_dir, ffado_scons_options, ffado_log))
if retval:
    print """
Failed to configure/build FFADO. Most likely this is due to uninstalled dependencies.
Check %s for details.
""" % ffado_log
    exit(-1)

# install FFADO
print " Installing into %s..." % sandbox_dir
retval = os.system('scons install >> %s' % (ffado_log))
if retval:
    print "Failed to install FFADO. Check %s for details." % ffado_log
    exit(-1)

# configure JACK
os.chdir("%s/jack/" % build_dir)
jack_log = "%s/jackbuild.log" % build_dir
os.system('echo "" > %s' % jack_log) 

print "Building Jack..."
if use_jack_version[1] == 'svn':
    print " Initializing build system..."
    retval = os.system('./autogen.sh >> %s' % jack_log)
    if retval:
        print """
Failed to initialize the jack build system. Most likely this is due to uninstalled dependencies.
Check %s for details.
""" % jack_log
        exit(-1)

print " Configuring build..."
retval = os.system('./configure --prefix="%s" >> %s' % (sandbox_dir, jack_log))
if retval:
    print """
Failed to configure the jack build. Most likely this is due to uninstalled dependencies.
Check %s for details.
""" % jack_log
    exit(-1)

# build and install jack
print " Compiling..."
retval = os.system('make >> %s' % (jack_log))
if retval:
    print "Failed to build jack. Check %s for details." % jack_log
    exit(-1)

print " Installing into %s..." % sandbox_dir
retval = os.system('make install >> %s' % (jack_log))
if retval:
    print "Failed to install jack. Check %s for details." % jack_log
    exit(-1)

# write the bashrc file
sandbox_bashrc = """
#!/bin/bash
#

LD_LIBRARY_PATH="%s/lib:$LD_LIBRARY_PATH"
PKG_CONFIG_PATH="%s/lib/pkgconfig:$PKG_CONFIG_PATH"
PATH="%s/bin:$PATH"

export LD_LIBRARY_PATH
export PKG_CONFIG_PATH
export PATH
""" % (sandbox_dir, sandbox_dir, sandbox_dir)

print "Writing shell configuration file..."
sandbox_rc_file = "%s/ffado.rc" % sandbox_dir
try:
    fid = open(sandbox_rc_file, "w")
    fid.write(sandbox_bashrc)
    fid.close()
except:
    print "Could not write the sandbox rc file."
    exit(-1)

os.chdir(cwd)

print """
FFADO and jack are installed into %s. If you want to use the
versions in the sandbox, you have to alter your environment
such that it uses them instead of the system versions.

If you use the bash shell (or compatible) you can use the following
rc script: %s. The procedure to use the sandboxed ffado+jack would be:

   $ source %s
   $ jackd -R -d firewire

If you don't use the bash shell, you have to alter your environment
manually. Look at the %s script for inspiration.

Note that you have to source the settings script for every terminal
you open. If you want a client application to use the sandboxed jack
you have to source the settings.

E.g.:
 terminal 1:
   $ source %s
   $ jackd -R -d firewire
 terminal 2:
   $ source %s
   $ ardour2
 terminal 3:
   $ source %s
   $ qjackctl &
   $ hydrogen

If you want to use the sandboxed version as default, you have to ensure
that the default environment is changed. This can be done e.g. by adding
the settings script to ~/.bashrc.

The build directory %s can now be deleted. To uninstall this sandbox, delete
the %s directory.
""" % (sandbox_dir, sandbox_rc_file, sandbox_rc_file, \
       sandbox_rc_file, sandbox_rc_file, sandbox_rc_file, \
       sandbox_rc_file, build_dir, sandbox_dir)



# coding=utf8
#
# Copyright (C) 2009 by Arnold Krille
#
# This file is part of FFADO
# FFADO = Free Firewire (pro-)audio drivers for linux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from PyQt4 import QtGui, QtCore, Qt
import dbus, math, decimal

import logging
log = logging.getLogger("matrixmixer")

class ColorForNumber:
    def __init__(self):
        self.colors = dict()

    def addColor(self, n, color):
        self.colors[n] = color

    def getColor(self, n):
        #print "ColorForNumber.getColor( %g )" % (n)
        keys = self.colors.keys()
        keys.sort()
        low = keys[-1]
        high = keys[-1]
        for i in range(len(keys)-1):
            if keys[i] <= n and keys[i+1] > n:
                low = keys[i]
                high = keys[i+1]
        #print "%g is between %g and %g" % (n, low, high)
        f = 0
        if high != low:
            f = (n-low) / (high-low)
        lc = self.colors[low]
        hc = self.colors[high]
        return QtGui.QColor(
                (1-f)*lc.red()   + f*hc.red(),
                (1-f)*lc.green() + f*hc.green(),
                (1-f)*lc.blue()  + f*hc.blue() )

class MixerNode(QtGui.QAbstractSlider):
    def __init__(self, input, output, value, max, muted, inverted, parent):
        QtGui.QAbstractSlider.__init__(self, parent)
        #log.debug("MixerNode.__init__( %i, %i, %i, %i, %s )" % (input, output, value, max, str(parent)) )

        self.pos = QtCore.QPointF(0, 0)
        self.input = input
        self.output = output
        self.setOrientation(Qt.Qt.Vertical)
        if max == -1:
            max = pow(2, 16)-1
        self.setRange(0, max)
        self.setValue(value)
        self.connect(self, QtCore.SIGNAL("valueChanged(int)"), self.internalValueChanged)

        self.setSmall(False)

        self.bgcolors = ColorForNumber()
        self.bgcolors.addColor(             0.0, QtGui.QColor(  0,   0,   0))
        self.bgcolors.addColor(             1.0, QtGui.QColor(  0,   0, 128))
        self.bgcolors.addColor(   math.pow(2,6), QtGui.QColor(  0, 255,   0))
        self.bgcolors.addColor(  math.pow(2,14), QtGui.QColor(255, 255,   0))
        self.bgcolors.addColor(math.pow(2,16)-1, QtGui.QColor(255,   0,   0))

        self.setContextMenuPolicy(Qt.Qt.ActionsContextMenu)
        self.mapper = QtCore.QSignalMapper(self)
        self.connect(self.mapper, QtCore.SIGNAL("mapped(const QString&)"), self.directValues)

        self.spinbox = QtGui.QDoubleSpinBox(self)
        self.spinbox.setRange(-40, 12)
        self.spinbox.setSuffix(" dB")
        self.connect(self.spinbox, QtCore.SIGNAL("valueChanged(const QString&)"), self.directValues)
        action = QtGui.QWidgetAction(self)
        action.setDefaultWidget(self.spinbox)
        self.addAction(action)

        for text in ["3 dB", "0 dB", "-3 dB", "-20 dB", "-inf dB"]:
            action = QtGui.QAction(text, self)
            self.connect(action, QtCore.SIGNAL("triggered()"), self.mapper, QtCore.SLOT("map()"))
            self.mapper.setMapping(action, text)
            self.addAction(action)

        # Only show the mute menu item if a value has been supplied
        self.mute_action = None
        if (muted != None):
            action = QtGui.QAction(text, self)
            action.setSeparator(True)
            self.addAction(action)
            self.mute_action = QtGui.QAction("Mute", self)
            self.mute_action.setCheckable(True)
            self.mute_action.setChecked(muted)
            self.connect(self.mute_action, QtCore.SIGNAL("triggered()"), self.mapper, QtCore.SLOT("map()"))
            self.mapper.setMapping(self.mute_action, "Mute")
            self.addAction(self.mute_action)

        # Similarly, only show a phase inversion menu item if in use
        self.inv_action = None
        if (inverted != None):
            if (muted == None):
                action = QtGui.QAction(text, self)
                action.setSeparator(True)
                self.addAction(action)
            self.inv_action = QtGui.QAction("Invert", self)
            self.inv_action.setCheckable(True)
            self.inv_action.setChecked(inverted)
            self.connect(self.inv_action, QtCore.SIGNAL("triggered()"), self.mapper, QtCore.SLOT("map()"))
            self.mapper.setMapping(self.inv_action, "Invert")
            self.addAction(self.inv_action)

    def directValues(self,text):
        #log.debug("MixerNode.directValues( '%s' )" % text)
        if text == "Mute":
            #log.debug("Mute %d" % self.mute_action.isChecked())
            self.update()
            self.parent().mutes_interface.setValue(self.output, self.input, self.mute_action.isChecked())
        elif text == "Invert":
            #log.debug("Invert %d" % self.inv_action.isChecked())
            self.update()
            self.parent().inverts_interface.setValue(self.output, self.input, self.inv_action.isChecked())
        else:
            text = text.split(" ")[0].replace(",",".")
            n = pow(10, (float(text)/20)) * pow(2,14)
            #log.debug("%g" % n)
            self.setValue(n)

    def mousePressEvent(self, ev):
        if ev.buttons() & Qt.Qt.LeftButton:
            self.pos = ev.posF()
            self.tmpvalue = self.value()
            ev.accept()
            #log.debug("MixerNode.mousePressEvent() %s" % str(self.pos))

    def mouseMoveEvent(self, ev):
        if hasattr(self, "tmpvalue") and self.pos is not QtCore.QPointF(0, 0):
            newpos = ev.posF()
            change = newpos.y() - self.pos.y()
            #log.debug("MixerNode.mouseReleaseEvent() change %s" % (str(change)))
            self.setValue( self.tmpvalue - math.copysign(pow(abs(change), 2), change) )
            ev.accept()

    def mouseReleaseEvent(self, ev):
        if hasattr(self, "tmpvalue") and self.pos is not QtCore.QPointF(0, 0):
            newpos = ev.posF()
            change = newpos.y() - self.pos.y()
            #log.debug("MixerNode.mouseReleaseEvent() change %s" % (str(change)))
            self.setValue( self.tmpvalue - math.copysign(pow(abs(change), 2), change) )
            self.pos = QtCore.QPointF(0, 0)
            del self.tmpvalue
            ev.accept()

    def paintEvent(self, ev):
        p = QtGui.QPainter(self)
        rect = self.rect()
        v = self.value()
        if (self.mute_action!=None and self.mute_action.isChecked()):
            color = QtGui.QColor(64, 64, 64)
        else:
            color = self.bgcolors.getColor(v)
        p.fillRect(rect, color)

        if self.small:
            return

        if color.valueF() < 0.6:
            p.setPen(QtGui.QColor(255, 255, 255))
        else:
            p.setPen(QtGui.QColor(0, 0, 0))
        lv=decimal.Decimal('-Infinity')
        if v != 0:
            lv = 20 * math.log10(v * 1.0 / math.pow(2,14))
            #log.debug("new value is %g dB" % lv)
        text = "%.2g dB" % lv
        if v == 0:
            text = "-ꝏ dB"
        p.drawText(rect, Qt.Qt.AlignCenter, QtCore.QString.fromUtf8(text))
        if (self.inv_action!=None and self.inv_action.isChecked()):
            p.drawText(rect, Qt.Qt.AlignLeft|Qt.Qt.AlignTop, QtCore.QString.fromUtf8(" ϕ"))

    def internalValueChanged(self, value):
        #log.debug("MixerNode.internalValueChanged( %i )" % value)
        if value is not 0:
            dB = 20 * math.log10(value / math.pow(2,14))
            if self.spinbox.value() is not dB:
                self.spinbox.setValue(dB)
        self.emit(QtCore.SIGNAL("valueChanged"), (self.input, self.output, value) )
        self.update()

    def setSmall(self, small):
        self.small = small
        if small:
            self.setMinimumSize(10, 10)
        else:
            fontmetrics = self.fontMetrics()
            self.setMinimumSize(fontmetrics.boundingRect("-0.0 dB").size()*1.1)
        self.update()


class MixerChannel(QtGui.QWidget):
    def __init__(self, number, parent=None, name="", smallFont=False):
        QtGui.QWidget.__init__(self, parent)
        layout = QtGui.QGridLayout(self)
        self.number = number
        if name != "":
            name = "\n(%s)" % name
        self.name = name
        self.lbl = QtGui.QLabel(self)
        self.lbl.setAlignment(Qt.Qt.AlignCenter)
        if (smallFont):
            font = self.lbl.font()
            font.setPointSize(font.pointSize()/1.5)
            self.lbl.setFont(font)
        layout.addWidget(self.lbl, 0, 0, 1, 2)
        self.hideChannel(False)

        self.setContextMenuPolicy(Qt.Qt.ActionsContextMenu)

        action = QtGui.QAction("Make this channel small", self)
        action.setCheckable(True)
        self.connect(action, QtCore.SIGNAL("triggered(bool)"), self.hideChannel)
        self.addAction(action)

    def hideChannel(self, hide):
        if hide:
            self.lbl.setText("%i" % self.number);
        else:
            self.lbl.setText("Ch. %i%s" % (self.number+1, self.name))
        self.emit(QtCore.SIGNAL("hide"), self.number, hide)
        self.update()



class MatrixMixer(QtGui.QWidget):
    def __init__(self, servername, basepath, parent=None, sliderMaxValue=-1, mutespath=None, invertspath=None, smallFont=False):
        QtGui.QWidget.__init__(self, parent)
        self.bus = dbus.SessionBus()
        self.dev = self.bus.get_object(servername, basepath)
        self.interface = dbus.Interface(self.dev, dbus_interface="org.ffado.Control.Element.MatrixMixer")

        self.mutes_dev = None
        self.mutes_interface = None
        if (mutespath != None):
            self.mutes_dev = self.bus.get_object(servername, mutespath)
            self.mutes_interface = dbus.Interface(self.mutes_dev, dbus_interface="org.ffado.Control.Element.MatrixMixer")

        self.inverts_dev = None
        self.inverts_interface = None
        if (invertspath != None):
            self.inverts_dev = self.bus.get_object(servername, invertspath)
            self.inverts_interface = dbus.Interface(self.inverts_dev, dbus_interface="org.ffado.Control.Element.MatrixMixer")

        #palette = self.palette()
        #palette.setColor(QtGui.QPalette.Window, palette.color(QtGui.QPalette.Window).darker());
        #self.setPalette(palette)

        cols = self.interface.getColCount()
        rows = self.interface.getRowCount()
        log.debug("Mixer has %i rows and %i columns" % (rows, cols))

        layout = QtGui.QGridLayout(self)
        self.setLayout(layout)

        self.rowHeaders = []
        self.columnHeaders = []
        self.items = []

        # Add row/column headers, but only if there's more than one 
        # row/column
        if (cols > 1):
            for i in range(cols):
                ch = MixerChannel(i, self, self.interface.getColName(i), smallFont)
                self.connect(ch, QtCore.SIGNAL("hide"), self.hideColumn)
                layout.addWidget(ch, 0, i+1)
                self.columnHeaders.append( ch )
            layout.setRowStretch(0, 0)
            layout.setRowStretch(1, 10)
        if (rows > 1):
            for i in range(rows):
                ch = MixerChannel(i, self, self.interface.getRowName(i), smallFont)
                self.connect(ch, QtCore.SIGNAL("hide"), self.hideRow)
                layout.addWidget(ch, i+1, 0)
                self.rowHeaders.append( ch )

        # Add node-widgets
        for i in range(rows):
            self.items.append([])
            for j in range(cols):
                mute_value = None
                if (self.mutes_interface != None):
                    mute_value = self.mutes_interface.getValue(i,j)
                inv_value = None
                if (self.inverts_interface != None):
                    inv_value = self.inverts_interface.getValue(i,j)
                node = MixerNode(j, i, self.interface.getValue(i,j), sliderMaxValue, mute_value, inv_value, self)
                if (smallFont):
                    font = node.font()
                    font.setPointSize(font.pointSize()/1.5)
                    node.setFont(font)
                self.connect(node, QtCore.SIGNAL("valueChanged"), self.valueChanged)
                layout.addWidget(node, i+1, j+1)
                self.items[i].append(node)

        self.hiddenRows = []
        self.hiddenCols = []


    def checkVisibilities(self):
        for x in range(len(self.items)):
            for y in range(len(self.items[x])):
                self.items[x][y].setSmall(
                        (x in self.hiddenRows)
                        | (y in self.hiddenCols)
                        )


    def hideColumn(self, column, hide):
        if hide:
            self.hiddenCols.append(column)
        else:
            self.hiddenCols.remove(column)
        self.checkVisibilities()
    def hideRow(self, row, hide):
        if hide:
            self.hiddenRows.append(row)
        else:
            self.hiddenRows.remove(row)
        self.checkVisibilities()

    def valueChanged(self, n):
        #log.debug("MatrixNode.valueChanged( %s )" % str(n))
        self.interface.setValue(n[1], n[0], n[2])

#
# vim: et ts=4 sw=4 fileencoding=utf8



#
# Copyright (C) 2008 Pieter Palmers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Test for common FFADO problems
#

import sys

import os
import commands
import re
import logging

## logging setup
logging.basicConfig()
log = logging.getLogger('diag')

## helper routines

# kernel
def get_kernel_version():
    return run_command('uname -r')

def get_kernel_rt_patched():
    l = run_command('uname -v')
    if l.find("PREEMPT RT") > -1:
        return True
    return False

def get_kernel_preempt():
    l = run_command('uname -v')
    if l.find(" PREEMPT ") > -1 and l.find(" RT ") == -1:
        return True
    return False

# modules
def check_for_module_loaded(modulename, procfile):
    log.info("Checking if module '%s' is present in %s... " % (modulename, procfile))
    f = open(procfile)
    lines = f.readlines()
    f.close()
    for l in lines:
        if l.find(modulename) > -1 or l.find(modulename.replace('-', '_')) > -1:
            log.info(" found")
            return True
    log.info(" not found")
    return False

def check_for_module_present(modulename):
    log.info("Checking if module '%s' is present... " % modulename)
    kver = get_kernel_version()
    (exitstatus, outtext) = commands.getstatusoutput("find \"/lib/modules/%s/\" -name '%s.ko' | grep '%s'" % \
                                                     (kver, modulename, modulename) )
    log.debug("find outputs: %s" % outtext)
    if outtext == "":
        log.info(" not found")
        return False
    else:
        log.info(" found")
        return True

def check_1394oldstack_active():
    return check_for_module_loaded('ohci1394', '/proc/interrupts')

def check_1394oldstack_linked():
    return os.access('/sys/module/ohci1394', os.F_OK) and \
           os.access('/sys/module/raw1394',  os.F_OK)

def check_1394oldstack_loaded():
    retval = True
    for modulename in ('ieee1394', 'ohci1394', 'raw1394'):
        if not check_for_module_loaded(modulename, '/proc/modules'):
            retval = False
    return retval

def check_1394oldstack_present():
    retval = True
    for modulename in ('ieee1394', 'ohci1394', 'raw1394'):
        if not check_for_module_present(modulename):
            retval = False
    return retval

def check_1394newstack_active():
    return check_for_module_loaded('firewire_ohci', '/proc/interrupts')

def check_1394newstack_linked():
    return os.access('/sys/module/firewire_ohci', os.F_OK)

def check_1394newstack_loaded():
    retval = True
    for modulename in ('firewire_core', 'firewire_ohci'):
        if not check_for_module_loaded(modulename, '/proc/modules'):
            retval = False
    return retval

def check_1394newstack_present():
    retval = True
    for modulename in ('firewire-core', 'firewire-ohci'):
        if not check_for_module_present(modulename):
            retval = False
    return retval

def check_1394oldstack_devnode_present():
    return os.path.exists('/dev/raw1394')

def check_1394oldstack_devnode_permissions():
    f = open('/dev/raw1394','w')
    if f:
        f.close()
        return True
    else:
        return False

def run_command(cmd):
    (exitstatus, outtext) = commands.getstatusoutput(cmd)
    log.debug("%s outputs: %s" % (cmd, outtext))
    return outtext

# package versions
def get_package_version(name):
    cmd = "pkg-config --modversion %s" % name
    return run_command(cmd)

def get_package_flags(name):
    cmd = "pkg-config --cflags --libs %s" % name
    return run_command(cmd)

def get_command_path(name):
    cmd = "which %s 2> /dev/null" % name
    return run_command(cmd)

def get_version_first_line(cmd):
    ver = run_command(cmd).split("\n")
    if len(ver) == 0:
        ver = ["None"]
    return ver[0]


def list_host_controllers():
    lspci_cmd = get_command_path("lspci")
    if lspci_cmd == "":
        lspci_cmd = "/sbin/lspci"
    cmd = lspci_cmd + " | grep 1394"
    controllers = run_command(cmd).split("\n")
    log.debug(lspci_cmd + " | grep 1394: %s" % controllers)
    for c in controllers:
        tmp = c.split()
        if len(tmp) > 0:
            tmp
            cmd = lspci_cmd + " -vv -nn -s %s" % tmp[0]
            print run_command(cmd)

def get_juju_permissions():
    return run_command('ls -lh /dev/fw*')

def get_user_ids():
    return run_command('id');



#
# Copyright (C) 2008 Pieter Palmers
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Help strings for ffado problems
#

class HelpStrings:
    def __init__(self):
        strings = {}
        strings['MODULES_OLD_STACK_NOT_INSTALLED']="""
FireWire kernel stack not present. Please compile the kernel with 
FireWire support.
"""
        strings['MODULES_BOTH_STACKS_LOADED']="""
Both old and new FireWire kernel modules are loaded, your system 
configuration is bogus.
"""
        strings['MODULES_NEW_STACK_LOADED']="""
The new FireWire kernel stack is loaded. 
If running a kernel earlier than 2.6.37 and problems are experienced, either 
try with the old Firewire kernel stack or upgrade to a newer kernel 
(preferrably 2.6.37 or later).
"""
        strings['MODULES_OLD_STACK_NOT_LOADED']="""
FireWire kernel module(s) not found. 
Please ensure that the raw1394 module is loaded.
"""
        strings['DEVNODE_OLD_STACK_NOT_PRESENT']="""
/dev/raw1394 device node not present. 
Please fix your udev configuration or add it as a static node.
"""
        strings['DEVNODE_OLD_STACK_NO_PERMISSION']="""
Not enough permissions to access /dev/raw1394 device. 
Please fix your udev configuration, the node permissions or
the user/group permissions.
"""

        self.strings=strings

    def get(self, sid):
        return self.strings[sid]

    def show(self, sid):
        print self.get(sid)
    
